/*********************************************** 
cmdparse.cpp

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, checking whether "value"
is actually a flag. Save flags and their values to array.
Flags may not start with a digit.

Definitions of Variables

External:
See generic.h


Internal:


***********************************************/
#pragma warning(disable:4786)

#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "dlmodule_includes.h"

extern char const*	CPP_Usage[MAX_FILENAME_LENGTH];

M_CmdParse::M_CmdParse(int argc_in, char **argv_in)
{
	/* Constructor */
	I_status = 0;
	argc = argc_in;
	argv = argv_in;
	I_Numflags = 0;
	flags = NULL;

}

M_CmdParse::~M_CmdParse()
{
/* Allocated in parse_args */
if(flags!=NULL) free(flags);
}

int M_CmdParse::GetStatus()
{
return I_status;
}

int M_CmdParse::GetArgc()
{
return argc;
}

char** M_CmdParse::GetArgv()
{
return argv;
}

void M_CmdParse::SetArgc(int argc_in)
{
argc = argc_in;
}

void M_CmdParse::SetArgv(char **argv_in)
{
argv = argv_in;
}


int M_CmdParse::parse_args()
{

int	i=1;

/* Run regardless of I_status so that
   at least log file name will be known.
   Uses own I_status since main status
   structure might not be declared yet. */
   

      while( i < argc )
        {
          if( argv[i][0] == '-' )
            {	    
		/* Allocate more memory for flags and value */
	    	flags = (M_Flag*) realloc(flags, (I_Numflags+1)*sizeof(M_Flag));
		if(flags==NULL)
		{I_status = PARSEARGS|MALLOC_FAILED; return(I_status);}
		sprintf(flags[I_Numflags].CP_Flag, "%s", argv[i]);
		sprintf(flags[I_Numflags].CP_FlagValue, "%s", "");
		
		if(i+1<argc)
		if(argv[i+1][0] != '-')
		sprintf(flags[I_Numflags].CP_FlagValue, "%s", argv[++i]);
		else
		if(i+1<argc)
		if(isdigit(argv[i+1][1]))
		sprintf(flags[I_Numflags].CP_FlagValue, "%s", argv[++i]);
		
		I_Numflags++;
	    
            }
          else
            {
		printf("parse_args: command line syntax error\n");
		I_status = PARSEARGS|SYNTAXERR;
		return(I_status);
            }
          i++;
        }

if(argc==1)	{
int	i=0;
while(CPP_Usage[i]) printf("%s", CPP_Usage[i++]);
I_status = PARSEARGS|SYNTAXERR;
}

if(0)
for(i=0;i<I_Numflags;i++)
printf("parse_args: flags = \t%s\tValue = \t%s\n",
flags[i].CP_Flag, flags[i].CP_FlagValue);

  return(I_status);
}


/*********************************************** 
get_flag_value

Purpose
Look up the value of the named variable in the 
command line arguments.

Overview
Loop through the table of values produced by
parse_args, seeking the designatated flag, and
overwrite the variable's value if the flag is found.

Definitions of Variables

External:
See generic.h


Internal:


***********************************************/


int M_CmdParse::get_flag_value(char const* CP_Flag, unsigned short& US_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sscanf(flags[i].CP_FlagValue, "%hd", &US_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %d\n", CP_Flag, US_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, short& S_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sscanf(flags[i].CP_FlagValue, "%hd", &S_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %d\n", CP_Flag, S_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, unsigned int& UI_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
if(strcmp(flags[i].CP_FlagValue, "") == 0)
UI_val = 1 - UI_val;
else
sscanf(flags[i].CP_FlagValue, "%d", &UI_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %d\n", CP_Flag, UI_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, int& I_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
if(strcmp(flags[i].CP_FlagValue, "") == 0)
I_val = 1 - I_val;
else
sscanf(flags[i].CP_FlagValue, "%d", &I_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %d\n", CP_Flag, I_val);
}
  return(0);
}

int M_CmdParse::get_flag_value(char const* CP_Flag, long& L_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
if(strcmp(flags[i].CP_FlagValue, "") == 0)
L_val = 1 - L_val;
else
sscanf(flags[i].CP_FlagValue, "%d", &L_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %d\n", CP_Flag, L_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, float& F_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sscanf(flags[i].CP_FlagValue, "%f", &F_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %f\n", CP_Flag, F_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, double& D_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sscanf(flags[i].CP_FlagValue, "%lf", &D_val);
if(0)printf("get_flag_value: Flag = %s\t New Value = %f\n", CP_Flag, D_val);
}
  return(0);
}


int M_CmdParse::get_flag_value(char const* CP_Flag, char*	CP_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sprintf(CP_val, "%s", flags[i].CP_FlagValue);
if(0)printf("get_flag_value: Flag = %s\t New Value = %s\n", CP_Flag, CP_val);
}
  return(0);
}

int M_CmdParse::get_flag_value(char* CP_Flag, char*	CP_val)
{
int		i;
for(i=0;i<I_Numflags;i++)
if(strcmp(CP_Flag, flags[i].CP_Flag) == 0)	{
sprintf(CP_val, "%s", flags[i].CP_FlagValue);
if(0)printf("get_flag_value: Flag = %s\t New Value = %s\n", CP_Flag, CP_val);
}
  return(0);
}
