/*********************************************** 
readwrite.cpp

Purpose
Read and write the FITS input and output files.

Overview
Loop over the number of input or output files,
calling the M_fitsinfo function for reading
or writing.

Definitions of Variables

External:
See generic.h


Internal:


***********************************************/

#include <string.h>
#include "dlmodule_includes.h"

extern const char*	CPP_Outfilenames[MAX_FILENAME_LENGTH];
extern const char*	CPP_NamelistNamesOut[MAX_FILENAME_LENGTH];
extern const char*	CPP_Infilenames[MAX_FILENAME_LENGTH];
extern const char*	CPPP_KeywordsIn[][FLEN_KEYWORD];
extern const char*	CPP_NamelistNamesIn[MAX_FILENAME_LENGTH];

M_ReadWrite::M_ReadWrite(	const int	I_NumInfilesIn,
				const int	I_NumOutfilesIn)
{
	I_NumInfiles = I_NumInfilesIn;
	I_NumOutfiles = I_NumOutfilesIn;

}

M_ReadWrite::~M_ReadWrite()
{

}


/*********************************************** 
read_inputs()

Purpose
Construct structures and read data for all input fits files.


Overview

Loop through input files: Call constructor. Save pointer into
array. Overwrite filenames if specified. Read in fits data.


Definitions of Variables 
See generic.h

***********************************************/

int M_ReadWrite::read_inputs(	M_CmdParse 	*GP_Cmd,
				M_Namelist	*GP_Name,
				M_FITSinfo 	*GP_Infiles,
				M_Status	*GP_Stat)
{
if(GP_Stat->I_status) return(GP_Stat->I_status);

	for(int i=0;i<I_NumInfiles;i++)	{
	char	CP_Flag[MAX_FILENAME_LENGTH];
	M_FITSinfo G_FITS = M_FITSinfo(CPP_Infilenames[i], CPPP_KeywordsIn[i]);
	GP_Infiles[i] = G_FITS;	
	GP_Name->get_namelist_value(CPP_NamelistNamesIn[i], GP_Infiles[i].CP_Filename);
	sprintf(CP_Flag, "%s%d", "-i", i+1);
	GP_Cmd->get_flag_value(CP_Flag, GP_Infiles[i].CP_Filename);
	if(strcmp(GP_Infiles[i].CP_Filename, "NONE") != 0)	
	GP_Infiles[i].read_data(GP_Stat);
	}

return(GP_Stat->I_status);
}



/*********************************************** 
write_outputs()

Purpose
Construct structures and write data for all output fits files.


Overview

Loop through output files: Call constructor. Save pointer into
array. Overwrite filenames if specified. Set pointer to array
that holds data. Load the desired output keywords into the
structure. Write fits keywords and data.


Definitions of Variables 
See generic.h

***********************************************/


int M_ReadWrite::write_outputs(	M_CmdParse 	*GP_Cmd,
				M_Logfile	*GP_Log,
				M_Namelist	*GP_Name,
				M_FITSinfo	*GP_Infiles,
				void		*GP_Const,
				void		*GP_Init,
		 		void		*GP_Comp,
				M_FITSinfo	*GP_Outfiles,
				M_Status	*GP_Stat)
{
int		 I_fits_return_status=0;
fitsfile 	*ffp_FITS_In=NULL;

if(GP_Stat->I_status) return(GP_Stat->I_status);

#if 0
/* Read the required fits input file so its keywords can be copied to outputs 
   There is always at least one required input file, and it is always named 
   from the flag -i1, and hence, even though it is alread closed at this
   point, its name is still stored in GP_Infiles[0].CP_Filename */
fits_open_file(	&(ffp_FITS_In),
		GP_Infiles[0].CP_Filename,
		READONLY,
		&I_fits_return_status);	
//printf("readwrite: Opening for reading \n%s\t0x%x\n", GP_Infiles[0].CP_Filename, I_fits_return_status);	
if(I_fits_return_status)
{GP_Stat->I_status = READ_IMAGE|FITS_MSG_MASK|I_fits_return_status;
 return(GP_Stat->I_status);}

 
	/* Set the output data file names and pointers to data, etc */
	for(int i=0;i<I_NumOutfiles;i++)	{
	char	CP_Flag[MAX_FILENAME_LENGTH];
	M_FITSinfo G_FITS = M_FITSinfo(CPP_Outfilenames[i]);
	GP_Outfiles[i] = G_FITS;
	GP_Name->get_namelist_value(CPP_NamelistNamesOut[i], GP_Outfiles[i].CP_Filename);
	sprintf(CP_Flag, "%s%d", "-o", i+1);
	GP_Cmd->get_flag_value(CP_Flag, GP_Outfiles[i].CP_Filename);
	GP_Outfiles[i].set_output_data(i,
	GP_Infiles, GP_Const, GP_Init, GP_Comp, GP_Outfiles, GP_Stat);
	}

	/* Load the keywords and write the data */
	for(int j=0;j<I_NumOutfiles;j++)	{	
	GP_Outfiles[j].load_std_keywords(GP_Log->CP_ModName, GP_Log->CP_Version, GP_Stat);
	GP_Outfiles[j].load_extra_keywords(j,
	GP_Log, GP_Name, GP_Infiles, GP_Const, GP_Init, GP_Comp, GP_Outfiles, GP_Stat);
	if(strcmp(GP_Outfiles[j].CP_Filename, "NONE") != 0)
	GP_Outfiles[j].write_data(ffp_FITS_In, GP_Stat);
	}

	
/* Close the required fits input file from which the keywords were read */
if(ffp_FITS_In!=NULL) fits_close_file(ffp_FITS_In, &I_fits_return_status);
/* If error occurred before close_file, return its code */
if(GP_Stat->I_status) return(GP_Stat->I_status);
/* If error occurred in close_file, return that code */	
if(I_fits_return_status)
GP_Stat->I_status = READ_IMAGE|FITS_MSG_MASK|I_fits_return_status;

return(GP_Stat->I_status);
}
#endif

	for(int i=0;i<I_NumOutfiles;i++)	{
	GP_Outfiles[i].write_output(	i,	
					GP_Cmd,
					GP_Log,
					GP_Name,
					GP_Infiles,
					GP_Const,
					GP_Init,
		 			GP_Comp,
					GP_Outfiles,
					GP_Stat);
	}
	
	return(GP_Stat->I_status);
}
