#include <stdio.h>
#include "mathc90.h"
#include <stdlib.h>
#include <math.h>
#include "fcrt.h"
#include <string.h>
#include <time.h>

double  getclipcorrfact(double );
double  alphafactor(double );

trimmedstatistics(double y[], long nstart, long n, double f1, 
                  double f2, int *trimfactflag, double *facttotptr,
                  double *mean, double *stddev, double *datascale) 
{

   long mid, nsamps1, nsamps2, istart, iend, jstart, jend, i, j;

   double fmean, fstddev, stddev2;

   double sampclippercen, fact, facttot, alpha;

   double Sn;

   double  getclipcorrfact(double );
   double  alphafactor(double );

   if (*trimfactflag == 0) {


      sampclippercen = 0.5 * (1.0 - f2) * 100.0;

      fact = getclipcorrfact(sampclippercen);

      /* facttot is the STD. DEV. correction due to the Gaussian trimming. */

      alpha = alphafactor(f2);

      *facttotptr = sqrt(fact * f2) * alpha;

      *trimfactflag = 1;


   }

   facttot = *facttotptr;

   datascale_estimate(y, n, &Sn);

   *datascale = Sn;

   if (f1 == 0.0) {
      if (n % 2 > 0) {
         mid = n / 2;
         *mean = y[mid];
      } else {
         mid = n / 2 - 1;
         *mean = (y[mid] + y[mid+1]) * 0.5;
      }

      nsamps2 = 0;
      *stddev = 0.0;
      stddev2 = 0.0;
      fstddev = 0.5 * (1.0 - f2);
      jstart = nstart - 1 + nint(fstddev * (n - nstart + 1));
      jend = n - 1 - nint(fstddev * (n - nstart + 1));

      for (j = jstart; j <= jend; j++) {
         *stddev += (y[j] - (*mean)) * (y[j] - (*mean));
         stddev2 += y[j] - (*mean);
         nsamps2++;
      }

      if (nsamps2 <= 1) {
         *stddev = 0.0;
      } else {
         *stddev = facttot * sqrt(1. / (nsamps2 - 1) *
            (*stddev - 1. / nsamps2 * stddev2 * stddev2));
      }

   } else {

      nsamps1 = 0;
      *mean = 0.0;
      fmean = 0.5 * (1.0 - f1);
      istart = nstart - 1 + (long) (fmean * (n - nstart + 1));
      iend = n - 1 - (long) (fmean * (n - nstart + 1));

      nsamps2 = 0;
      *stddev = 0.0;
      stddev2 = 0.0;
      fstddev = 0.5 * (1.0 - f2);
      jstart = nstart - 1 + (long) (fstddev * (n - nstart + 1));
      jend = n - 1 - (long) (fstddev * (n - nstart + 1));

      for (i = istart; i <= iend; i++) {
         *mean += y[i];
         nsamps1++;
      }
      *mean /= nsamps1;

      for (j = jstart; j <= jend; j++) {
         *stddev += (y[j] - (*mean)) * (y[j] - (*mean));
         stddev2 += y[j] - (*mean);
         nsamps2++;
      }

      if (nsamps2 <= 1) {
         *stddev = 0.0;
      } else {
         *stddev = facttot * sqrt(1. / (nsamps2 - 1) *
            (*stddev - 1. / nsamps2 * stddev2 * stddev2));
      }
   }            
}
double alphafactor(double fcentral) {

   long i, n;

   double alphafactor, f[6], a[6], slope, yint;

   f[0] = 0.0;
   f[1] = 0.2;
   f[2] = 0.4;
   f[3] = 0.6;
   f[4] = 0.8;
   f[5] = 1.0;

   a[0] = 1.253;
   a[1] = 1.184;
   a[2] = 1.126;
   a[3] = 1.080;
   a[4] = 1.036;
   a[5] = 1.000;

   n = 6;

   alphafactor = f[5];

   for (i = 1; i <  n; i++) {
      if (fcentral <= f[i]) {
         slope = (a[i] - a[i - 1])/(f[i] - f[i - 1]);
         yint = a[i] - slope * f[i];
         alphafactor = slope * fcentral + yint;
         break;
      } 
   }

   return alphafactor;
 
}


datascale_estimate(double x[], long n, double *Sn) {

   int	i, k, tryA, tryB, diff, Amin, Amax, even, half;
   int leftA, leftB, rightA, rightB, nA, nB, length;

   long nstart;

   double *y, *a2, medA, medB, cn, pull;

   y = (double *) calloc(n + 1000, sizeof(double));

   a2 = (double *) calloc(n + 1000, sizeof(double));

   nstart = 1;
   dsort(x, nstart, n);

   for (i = n; i >= 1 ; i--) {
      y[i] = x[i-1];
   }

   a2[1] = y[n/2+1] - y[1];

   for (i = 2; i <= (n + 1) / 2; i++) {

      nA = i - 1;
      nB = n - i;
      diff = nB - nA;
      leftA = 1;
      leftB = 1;
      rightA = nB;
      rightB = nB;
      Amin = diff / 2 + 1;
      Amax = diff / 2 + nA;

loopback:

      if (leftA < rightA) {
         length = rightA - leftA + 1;
         even = 1 - length % 2;
         half = (length - 1) / 2;
         tryA = leftA + half;
         tryB = leftB + half;
         if (tryA < Amin) {
            rightB = tryB;
            leftA = tryA + even;
         } else {
	    if (tryA > Amax) {
               rightA = tryA;
               leftB = tryB + even;           
	    } else {
               medA = y[i] - y[i - tryA + Amin - 1];
               medB = y[tryB + i] - y[i];
               if (medA >= medB) {
                  rightA = tryA;
                  leftB = tryB + even;
               } else {
                  rightB = tryB;
                  leftA = tryA + even;
               }
            }
         }
         goto loopback;
      }
      if (leftA > Amax) {
         a2[i] = y[leftB + i] - y[i];
      } else {
         medA = y[i] - y[i - leftA + Amin - 1];
         medB = y[leftB + i] - y[i];

         a2[i] = medA;
         if (medB < medA) a2[i] = medB;
      }
   }

   for (i = (n + 1) / 2 + 1; i <= n - 1; i++) {
      nA = n - i;
      nB = i - 1;
      diff = nB - nA;
      leftA = 1;
      leftB = 1;
      rightA = nB;
      rightB = nB;
      Amin = diff / 2 + 1;
      Amax = diff / 2 + nA;

loopback2:

      if (leftA < rightA) {
         length = rightA - leftA + 1;
         even = 1 - length % 2;
         half = (length - 1) / 2;
         tryA = leftA + half;
         tryB = leftB + half;
         if (tryA < Amin) {
            rightB = tryB;
            leftA = tryA + even;
         } else {
	    if (tryA > Amax) {
               rightA = tryA;
               leftB = tryB + even;           
	    } else {
               medA = y[i + tryA - Amin + 1] - y[i];
               medB = y[i] - y[i - tryB];
               if (medA >= medB) {
                  rightA = tryA;
                  leftB = tryB + even;
               } else {
                  rightB = tryB;
                  leftA = tryA + even;
               }
            }
         }
         goto loopback2;
      }
      if (leftA > Amax) {
         a2[i] = y[i] - y[i - leftB ];
      } else {
         medA = y[i + leftA - Amin + 1] - y[i];
         medB = y[i] - y[i - leftB];
         a2[i] = medA;
         if (medB < medA) a2[i] = medB;
      }
   }
   a2[n] = y[n] - y[(n + 1) / 2];
   cn = 1.0;
   if (n <= 9) {
      if (n == 2) cn = 0.743;
      if (n == 3) cn = 1.851;
      if (n == 4) cn = 0.954;
      if (n == 5) cn = 1.351;
      if (n == 6) cn = 0.993;
      if (n == 7) cn = 1.198;
      if (n == 8) cn = 1.005;
      if (n == 9) cn = 1.131;
   } else {
      if (n % 2 == 1) 
         cn = ((double) n) / ((double) n - 0.9);
   }

   free(y);

   k = (n + 1) / 2;
   pull_datascale_estimate(a2, n, k, &pull);

   *Sn = cn * 1.1926 * pull;

   free(a2);

}

double gausspdf(double x) {

   double gausspdf, pi;

   pi = 3.14159265358979323846;

   gausspdf = exp(-0.5 * x * x) / sqrt(2.0 * pi);

   return gausspdf;
 
}

double getclipcorrfact(double sampclippercen) {

   double sum,fact;
   double bigf,xbot,xtop;

   bigf=sampclippercen*0.01;

   if (bigf > 0.0) {

      if (bigf < 1.0) {
            getlimits(bigf,&xbot,&xtop);
            integrategauss(xbot,xtop,&sum);
            fact = 1.0/sum;
      } else {
         fact = 0.0;
      }

   } else {
      fact=1.0;
   }

      return fact;
}

getlimits(double bigf, double *xbot, double *xtop) {
  int n, i;
  double a, b, step, halfstep, x, x2, rest, sum;

  rest = 1.0 - bigf;
  a = 0.0;
  b = 4.0;
  n = 100000;
  step = 4.0 / (double) n;
  halfstep = 0.5 * step;
  sum = 0.5;
  
  for (i = 1; i <= n - 1; i++) {
    x = a + ((double)(i-1)) * step;
    x2 = x + step;
    sum += (gausspdf(x) + gausspdf(x2)) * halfstep;
    if (sum >= rest) { break; }
  }
  *xbot = -x2;
  *xtop = x2;
}

integrategauss(double xbot, double xtop, double *sum) {

      int n, i;

      double step, halfstep, x;

      n = 100000;
      step = (xtop - xbot) / (double) n;
      halfstep = 0.5 * step;

      *sum = gausspdf(xbot) + gausspdf(xtop);

      for (i = 2; i <= n - 1; i++) {
         x = xbot + (double)i * step;
         *sum += gausspdf(x) * 2.0 * x * x;
      }

      *sum *= halfstep;

}

pull_datascale_estimate(double a[], long n, long k, double *pull) {

   /* Finds the kth-order statistic. */

   int j, l, lr, jnc;

   double *b, ax, buffer;

   b = (double *) calloc(n + 1000, sizeof(double));

   for (j = 1; j <= n; j++) {
      b[j] = a[j];
   }

   l = 1;
   lr = n;
 branch20:
   if (l >= lr) 
      goto branch90;
   ax = b[k];
   jnc = l;
   j = lr;
 branch30:
   if (jnc > j) 
      goto branch80;
 branch40:
   if (b[jnc] >= ax) 
      goto branch50;
   jnc += 1;
   goto branch40;
 branch50:
   if (b[j] <= ax) 
      goto branch60;
   j -= 1;
   goto branch50;
 branch60:
   if (jnc > j) 
      goto branch70;
   buffer = b[jnc];
   b[jnc] = b[j];
   b[j] = buffer;
   jnc += 1;
   j -= 1;
 branch70:
   goto branch30;
 branch80:
   if (j < k)
      l = jnc;
   if (k < jnc)
      lr = j;      
   goto branch20;
 branch90:
   *pull = b[k];

   free(b);

}
