/* close.c */
/* %W% %E% %U% */
/* #Copyright (C) 1992, California Institute of Technology. */
/* #U.S. Government Sponsorship under NASA Contract NAS7-918 */
/* #is acknowledged. */

#include "im.h"
#include <stdio.h>

extern struct fid_struct *f;

static raw_pix_read(buffer,n_pix_bytes)
char *buffer;
int n_pix_bytes;
{
  int n_bytes;

  if(! f->unix_pix_fid) return 0;
  lseek(f->unix_pix_fid,f->io_offset,0);
  n_bytes = read(f->unix_pix_fid,buffer,n_pix_bytes);
  return(n_bytes);
}

im_close(fid)
int fid;
{

   struct header_struct *h;
   struct hdr_blk_struct *block, *next_block;
   char buffer[2880];
   char *zeroes;
   int n_pad, b;
   int pad;
   int stat;
   int total_pix_bytes;
   int current_hdr_sz;
   char *pixels;

   if (IM_set_fid(fid)) return(-3);

   switch(f->mode) {

   case 1:  /* read */
    break;

   case 2:  /* write */
   case 3:  /* update */
     total_pix_bytes = f->n_pixels * f->bytes_per_pix;
     current_hdr_sz = ((f->n_headers+N_MANDATORY_CARDS+35)/36) * 2880;
     /*printf("setup=%d, type=%d, mode=%d\n",
	    f->need_wpix_setup,f->file_type,f->mode);
     printf("currsz=%d, disksz=%d, io=%d\n",
     current_hdr_sz,f->written_hdr_size,f->io_offset);*/
     if (f->need_wpix_setup != 0) im_wpix_r(f,pad,0); /* force initialization */
    /* pad out pixel area of file if FITS */
     if (f->file_type == Type_fits) {
       if(f->mode == 3) {  /* Stuff to do when updating FITS files  */
	 if(current_hdr_sz != f->written_hdr_size) {
	   /* For updated FITS files, check to see if the header has changed    */
	   /* size from what is on disk. If so, we must read and re-write       */
	   /* the pixels to move them to a new offset, accomodating the new hdr */
	   /*printf("... rewriting\n");*/
	   if( ! (pixels = (char *) calloc(1,total_pix_bytes))) {
	     fprintf(stderr,"im_close: Failure to allocate pixel space.\n");
	     return(-3);
	   }
	   if((stat=raw_pix_read (pixels,total_pix_bytes))<0) return(stat);
	   /* Update hdr offset */
	   f->io_offset = current_hdr_sz;
	   if((stat=raw_pix_write(pixels,total_pix_bytes))<0) return(stat);
	   free(pixels);
	 }
       }
       /* If we need to fill-out a block, pad with zeros */
       if ((pad = total_pix_bytes % 2880)) {
	 pad = 2880 - pad;
	 zeroes = (char *) calloc(1,pad);
	 if (f->unix_pix_fid) {
	   /* Skip to bottom of pixels */
	   lseek(f->unix_pix_fid,f->io_offset+total_pix_bytes,0);
	   stat=write(f->unix_pix_fid,zeroes,pad);
	   if (stat <0) return(stat);
	 }
	 else fwrite(zeroes,1,pad,stdout);
	 free(zeroes);
       }
     }

     if (f->unix_pix_fid) {
       stat = IM_write_hdr();
       if (stat <0) return(stat);
     }
     break;


   default:  /* error */
     return(-3);

   }; /* end switch */

   close(f->unix_pix_fid);
   f->unix_pix_fid = -1;
   f->mode = 0;
   f->magic = 0;
   for (block=f->header_blocks; block;) {
      next_block = block->next;
      free(block);
      block = next_block;
   }
   free(f);

   return(0);
}

raw_pix_write(buffer,n_pix_bytes)
char *buffer;
int n_pix_bytes;
{
   int n_bytes;

  if(! f->unix_pix_fid) return 0;
  lseek(f->unix_pix_fid,f->io_offset,0);
  n_bytes = write(f->unix_pix_fid,buffer,n_pix_bytes);
  return(n_bytes);
}
