/* im.h */
/* %W% %E% %U% */
/* #Copyright (C) 1992, California Institute of Technology. */
/* #U.S. Government Sponsorship under NASA Contract NAS7-918 */
/* #is acknowledged. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define IM_MAGIC 0x7B01

#define N_HDRS_PER_BLK 50
#define MAX_WINDOWS 5

#define N_MANDATORY_CARDS 3

#define FLNAMSZ 257

struct header_struct { /* header cards are in a linked list */
   char text[81];
   struct header_struct *next; /* pointer to next header card */
   struct header_struct *prev; /* pointer to previous header card */
};

struct hdr_blk_struct {
   struct header_struct headers[N_HDRS_PER_BLK];
   struct hdr_blk_struct *next;
};

struct window_struct {
   int naxis_skip[9];
   int naxis_take[9];
   int left_to_read[9];
   int left_to_write[9];
   int start_byte;
   int total_bytes;
   int line_bytes;
   int skip_bytes;
   int cur_byte_read;
   int total_bytes_left_to_read;
   int line_bytes_left_to_read;
   int cur_byte_write;
   int total_bytes_left_to_write;
   int line_bytes_left_to_write;
   int axis_len_real[10];  /* index to real (naxisn>1) axes  [0]=# */
}; 

struct fid_struct {
   int mode; /* closed=0, read=1, write=2, update=3 */
   enum {Type_unknown, Type_fits, Type_zz, Type_iraf, Type_S} file_type;
   char user_filename[FLNAMSZ];
   char hdr_filename[FLNAMSZ];
   char pix_filename[FLNAMSZ];
   char subset_text[64];
   int unix_hdr_fid;
   int unix_pix_fid;
   int written_hdr_size;

   int bytes_per_pix; /* 1, 2, 4 , 8 */
   int n_pixels; /* product of all the NAXIS's */
   int io_offset; /* seek position to first pixel in file */
   int nan_flag; /* non-zero if all non-finites should convert to +NaN */
   int line_skip_pix; /* number of pad pixels at end of each line (IRAF) */
   int need_wpix_setup; /* write mode and setup needed before wpix */
   struct hdr_blk_struct *header_blocks;

   /* binary values from the header ... defaults in { }  */
   int val_bitpix;     /* {0} value of BITPIX keyword */
   int val_naxis[10];  /* {0} [0]=number of axes, [n>0]=length of axis */
   int val_blank_flag; /* {0} 0=none   1=yes  */
   int val_blank;      /* {0} value of BLANK keyword */
   double val_scale;    /* {1.} value of BSCALE keyword */
   double val_offset;   /* {0.} value of BZERO keyword */

   int n_headers;
   struct header_struct *cur_header; /* pointer to "current" header card */
   struct header_struct *first_header; /* pointer to first header card */
   struct header_struct *last_header ; /* pointer to last header card */
   struct header_struct *free_header ; /* pointer to last header card */

   struct window_struct *cur_window;
   struct window_struct windows[MAX_WINDOWS];

   int magic; /* magic number */
};
