/* wkey.c  (write keyword value) */
/* %W% %E% %U% */
/* #Copyright (C) 1992, California Institute of Technology. */
/* #U.S. Government Sponsorship under NASA Contract NAS7-918 */
/* #is acknowledged. */

#include "im.h"

static char text[120];
extern struct fid_struct *f;

/******* *******
 return codes:
-3 = bad fid
-2 = illegal keyword or value
 0 = ok, but not a real header card
 n = ok: number of header card put into
******* *******/

/*---------------------------------------------------------*/

static set_kwd(fid,keyword)
int fid;
char keyword[];
/* returns: >0 if OK, 0 if not a real card,   -2 if error,   -3 if bad fid */
{
   int stat;
   char card[80];

   if (IM_set_fid(fid)) return(-3);

   sprintf(card,"%-8.8s= %s",keyword,text);

   stat = IM_card_find(keyword);
   if (stat>0) stat = IM_card_replace(card);
   else {
      stat = IM_card_add(card);
   }

   return(stat);
}
/*---------------------------------------------------------*/

im_wkey_i(fid,value,keyword)
int fid;
int value;
char keyword[];
{
   sprintf(text,"%20d",value);
   return(set_kwd(fid,keyword));
}

/*---------------------------------------------------------*/

im_wkey_u(fid,value,keyword)
int fid;
unsigned long int value;
char keyword[];
{
   sprintf(text,"%20lu",value);
   return(set_kwd(fid,keyword));
}

/*---------------------------------------------------------*/

im_wkey_r(fid,value,keyword)
int fid;
float value;
char keyword[];
{
   char *c;
   int stat;
   int n;

   sprintf(text," %19.10G",value);

   for (c=text; *c && *c!='.'; c++);
   /* add decimal if none */
   if (*c=='\0') {
      for (c=text+1; *c; c++) *(c-1) = *c; *(c-1) = '.';
   }
   stat = set_kwd(fid,keyword);
   return(stat);
}

/*---------------------------------------------------------*/

im_wkey_d(fid,value,keyword)
int fid;
double value;
char keyword[];
{
   char *c;
   sprintf(text," %19.12G",value);
   for (c=text; *c && *c!='.'; c++);
   /* add decimal if none */
   if (*c=='\0') {for (c=text+1; *c; c++) *(c-1) = *c; *(c-1) = '.';}
   return(set_kwd(fid,keyword));
}

/*---------------------------------------------------------*/

im_wkey_c(fid,ptr,keyword)
int fid;
char *ptr;
char keyword[];
{

   if (ptr == 0) return(-2);

   sprintf(text,"'%-8.68s'",ptr);
   return(set_kwd(fid,keyword));
}

/*---------------------------------------------------------*/

im_wkey_l(fid,value,keyword)
int fid;
int value;
char keyword[];
{
   if (value) strcpy(text,"                   T");
   else       strcpy(text,"                   F");
   return(set_kwd(fid,keyword));
}

/*---------------------------------------------------------*/

im_wkey_t(fid,ptr,keyword)
int fid;
char *ptr;
char keyword[];
{
   int n;
   char *c, *j;
   char *card;
   int flag = 0; /* set !=0 if in a quoted string */

   if (IM_set_fid(fid)) return(-3);

   switch(keyword[0]) {

      case '/': /* comments */
       card = f->cur_header->text;
       c = card+10;
       /* &&& TPC changed this around a bit to:                    */
       /*     1) Not chop off a quote when adding a comment        */
       /*     2) Handle '...''...' strings                         */
       /*     3) Handle adding comments when comments exist already */
       while (*c!='\0' && (*c!='/' || flag==1) && c-card<79) {
          if (*c == '\'') {
	    if (flag) {
	      if(*(c+1) != '\'') flag=0; /* Allow '...''...' strings */
	      else               c++;   /* Skip past second quote */
	    }
	    else flag=1;
	  }
          c++; 
       }
       if(c-card > 75) return((int) f->cur_header); /* Can't add comment */
       while (c < card+30) *c++ = ' '; /* don't start before col 30 */
       if(*c == '/') c++;
       else { *c++ = ' '; *c++ = '/'; } 
       *c++ = ' ';
       n = card - c + 80;
       strncpy(c,ptr,n);
       c = card+80;
       *c = '\0';
       return((int) f->cur_header);

      case 'e': /* add new card at end */
         f->cur_header = f->last_header;

      case 'a': /* add after current card */
         return(IM_card_add(ptr));

      case 'r': /* replace current card */
         return(IM_card_replace(ptr));

      case 'd': /* delete current card */
         return(IM_card_delete());

   };

   return(-2);
}


