/****************************************************************************
** Routine Name:                isis_barparse.c                            **
**                                                                         **
** Description:                 This routine parses column using vertical  **
**				bar.					   **
**                                                                         **
** Input Parameters:            ifile   ____ input file number.            **
**			   	ncols   ____ column number 		   **
**			   	i       ____ i th char in a string         **	
**			   	*k      ____ k index in a string           **	
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      stat = TBL_OK (0)	success		   **
**                              stat = TBL_NOMEM (-2)   no memory left     **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    (1) Seperate from original code tbl_open.c **
**                              (2) Modify code                            **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


int isis_barparse(ifile, ncols, i, k)

int 	ifile;		/* input file number */
int 	ncols;		/* column index for an input line */
int 	i;		/* i th char in an input line */
int 	*k;		/* k th char of a name field */


/*************************
* define Local Variables *
**************************/
{
   int   name_size; 	/* size of name defined in colstruct */
   int   j;		/* loop index */
   char *rec;		/* current record information */ 


/**********************
* Parse Column String *
***********************/

   rec  = tbl[ifile]->rec;

   /* Copy char from an input line to a column name string */
   tbl[ifile]->colinfo[ncols]->name[*k]	= rec[i];	
   tbl[ifile]->colinfo[ncols]->name[*k]	= rec[i];	
   ++(*k);				/* next char  */

   /* Check memory to see if it requires more */
   if (*k == tbl[ifile]->colinfo[ncols]->maxname)
   {

      /* Allocate Memory Size for Name field */
      name_size	= tbl[ifile]->colinfo[ncols]->maxname 
		* sizeof(char);
      tbl[ifile]->colinfo[ncols]->name 
		= (char *) realloc(tbl[ifile]->colinfo[ncols]->name,
		   name_size); 

      if (tbl[ifile]->colinfo[ncols]->name == (char*)NULL)
      {
 	  sprintf(tblerror,"Cannot malloc () enough memory.");
	  return (TBL_NOMEM);
      }

      /* Reinitialization */
      for (j=tbl[ifile]->colinfo[ncols]->maxname - TBL_MAXHSTR;
	   j<tbl[ifile]->colinfo[ncols]->maxname; ++j )
	   tbl[ifile]->colinfo[ncols]->name[j] = '\0';

    } 	/* end of if statement */

    return(TBL_OK);
}
