/****************************************************************************
**                                                                         **
** Routine Name:                isis_close.c                               **
**                                                                         **
** Description:                 This routine closes a table file and       **
**                              free memory.                               **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      stat = TBL_OK (0),     sucess              **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:   	None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    Add header and comments                    **
**   2004-08    Iffat Khan      Modified to append in input file           **
**   2005-10    Iffat Khan      Modified to free allocated memory          **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/**************************
* Define Global Variables *
***************************/
int	maxfiles;		/* maximun number of files are allocated */
char    ***tbl_subtit;		/* contain header information */
int     tmp_output[TBL_MAXFILES];


isis_close(ifile)
int 	ifile;			/* input file number */	 


{
  /*************************
  * Define Local Variables *
  **************************/
  int	i, j, k;		/* loop index */
  int	ipac;			/* file index */
  int	icol;			/* column number */
  FILE  *tf;			/* current file */
  char  str[TBL_MAXSTR];

  /****************************
  * Initialize Local Pointers *
  *****************************/

  ipac  = tbl_finfo[ifile]->file_handle;
  tf 	= tbl[ipac]->tfile;


  /*************************
  * Check if it is a right * 
  * file to close          *
  **************************/

  if (ipac < 0 || ipac >= maxfiles)
  {
      sprintf(tblerror, "NO such file reference. ");
      return(TBL_ERROR);
  }

  if (tf == (FILE *) NULL)
  {
      sprintf(tblerror, "File not open.");
      return(TBL_ERROR);
  }

  /*****************
  * Close the file *
  ******************/
  fclose(tf);


  /*****************************
  * Reinitialize the variables *
  ******************************/
if (tbl_subtit != (char ***) NULL)
  for (i = 0; i < TBL_MAXSUBTIT; ++i)
    for (j = 0; j < tbl[ipac]->ncols; ++j)
      for (k = 0; k < TBL_MAXSUBLEN; ++k)
	tbl_subtit[i][j][k]='\0';

  tbl[ipac]->tfile 	= (FILE *) NULL;
  tbl[ipac]->header 	= 0;
  tbl[ipac]->reclen 	= 0;
  tbl[ipac]->ncols 	= 0;
  tbl[ipac]->keynum 	= 0;


  /************************
  * Free Allocated Memory *
  *************************/
  
  /* free rec memory */
  if (tbl[ipac]->rec) 
      free(tbl[ipac]->rec);

  if (tbl[ipac]->delrow) 
      free(tbl[ipac]->delrow);

  /* free column related memory */
  for (i=0; i<tbl[ipac]->maxcols; ++i)
  {
    if (tbl[ipac]->colinfo[i]->name)
	free(tbl[ipac]->colinfo[i]->name);
    if (tbl[ipac]->colinfo[i]->dtyp)
	free(tbl[ipac]->colinfo[i]->dtyp);
    if (tbl[ipac]->colinfo[i]->unit)
	free(tbl[ipac]->colinfo[i]->unit);
    if (tbl[ipac]->colinfo[i]->null)
	free(tbl[ipac]->colinfo[i]->null);
    if (tbl[ipac]->colinfo[i]->scal)
	free(tbl[ipac]->colinfo[i]->scal);
    if (tbl[ipac]->colinfo[i]->zero)
	free(tbl[ipac]->colinfo[i]->zero);

    free(tbl[ipac]->colinfo[i]);
  }
  free(tbl[ipac]->colinfo);

  /* free column value (used for writing out) */
  if(tbl[ipac]->colvalue)
  {
    for (i=0; i<tbl[ipac]->maxcols; ++i)
    {
      if (tbl[ipac]->colvalue[i])
      {
          if (tbl[ipac]->colvalue[i]->cptr)
            free(tbl[ipac]->colvalue[i]->cptr);
      
          if (tbl[ipac]->colvalue[i]->iptr)
            free(tbl[ipac]->colvalue[i]->iptr);
      
          if (tbl[ipac]->colvalue[i]->dptr)
            free(tbl[ipac]->colvalue[i]->dptr);

         tbl[ipac]->colvalue[i]->nelem = 0;
         free(tbl[ipac]->colvalue[i]);
      }
    }
    free (tbl[ipac]->colvalue);
  }
         
  tbl[ipac]->maxcols 	= 0;

  /* free keyword related memory */
  for (i=0; i<TBL_KEYNUM; ++i)
  {
    if (tbl[ipac]->keyinfo[i])
       free(tbl[ipac]->keyinfo[i]);
  }
  free(tbl[ipac]->keyinfo);
  return(TBL_OK);
}

