/***************************************************************************
**                                                                         **
** Routine Name:                isis_colinfo.c                             **
**                                                                         **
** Description:                 This routine retrieves a column information**
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              icol    ____ column number                 **
**                                                                         **
** Output Parameters:           **name  ---- column content                **
**                              *width  ____ width of a column record      **
**                              *endcol ____ ending column number of a     **
**                                           column record                 **
**                              **dtyp  ____ data type                     **
**                              **unit  ____ unit of the column            **
**                              **null  ____ null value of the column      **
**                              **scal  ____ scaling factor                **
**                              **zero  ____ offset                        **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:          None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    modification of the code by including      ** 
**				unit, type, scaling and null information   **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Includeing Files *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


isis_colinfo(ifile, icol, name, width, endcol,
	     dtyp, unit, null, scal, zero) 

int 	ifile;			/* input file number */	 
int 	icol;			/* column index */	 
int 	*width;			/* width of a column record */	 
int 	*endcol;	        /* ending column number */	 
char 	**name;			/* column name */	 
char 	**dtyp;			/* column datatype */	 
char 	**unit;			/* width of a column record */	 
char 	**null;			/* width of a column record */	 
char 	**scal;			/* width of a column record */	 
char 	**zero;			/* width of a column record */	 

/************************
* Define Local Variable *
*************************/
{
  FILE  *tf;			
  int	stat;
  int	ipac;

/***************************
* Initialize Local Pointer *
****************************/

  ipac  = tbl_finfo[ifile]->file_handle;
  tf 	= tbl[ipac]->tfile;


/*********************
* Check Column index *
**********************/

  if (icol<0 || icol>= tbl[ipac]->ncols)	/* column out of boundary */
  {
      sprintf(tblerror, " NO such column. ");	
      return (TBL_NOCOL);			
  }


/*******************
* Copy Information *
********************/

  *width  = tbl[ipac]->colinfo[icol]->width;	
  *endcol = tbl[ipac]->colinfo[icol]->endcol;	

  *name	  = tbl[ipac]->colinfo[icol]->name; 	
  *dtyp   = tbl[ipac]->colinfo[icol]->dtyp;	
  *unit   = tbl[ipac]->colinfo[icol]->unit;	
  *null   = tbl[ipac]->colinfo[icol]->null;	
  *scal   = tbl[ipac]->colinfo[icol]->scal;	
  *zero   = tbl[ipac]->colinfo[icol]->zero;	

  return(TBL_OK);
}

