/****************************************************************************
**							                   **	
** Routine Name:                isis_colmallc.c                            **
**                                                                         **
** Description:                 This routine allocates initial column space**
**                              and initializes associated fields with     **
**                              their default values defined in            **
**		                global_cnst.h.                             **
**                                                                         **
** Input Parameters:            ifile   ____ file number                   **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      TBL_OK    ( 0),  success                   **
**				TBL_NOMEM (-2),  no enough memory          **   
**                                                                         **
** Routine Be Called:		None					   **	
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    (1) Seperate this part from original       **
**				    tbl_open.c.    			   **
**                              (2) Modify code.                           **
**
**  2004-02     Iffat Khan      modified for colvalstruct
**  2005-10     Iffat Khan      initialized  colvalstruct pointers         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


int isis_colmallc(ifile)

int ifile;		   /* file number */ 

{
   int	 colstrc_size;     /* size of colstruct */
   int	 colvalstrc_size;  /* size of colvalstruct */
   int   name_size; 	   /* size of name defined in colstruct */
   int   value_size; 	   /* size of name defined in colstruct */
   int   i, j, l;	   /* loop index */


/************************************
*  Allocate Total Column Info Space *
*************************************/

   /* Initialize Number of  Columns */
   tbl[ifile]->maxcols 	= TBL_MAXCOLS;	
   tbl[ifile]->ncols 	= 0;	
   tbl[ifile]->ncols 	= 0;	

   /* Allocate Memory Size for A colstruct */
   colstrc_size		= sizeof(struct colstruct *);
   tbl[ifile]->colinfo 	= (struct colstruct **) 
			  malloc(tbl[ifile]->maxcols 
			* colstrc_size);

   if (tbl[ifile]->colinfo ==(struct colstruct **) NULL)
   {
       sprintf(tblerror,"Cannot malloc () enough memory.");
       return (TBL_NOMEM);
   }

   /* Allocate Memory Size for A colvalstruct */
   colvalstrc_size		= sizeof(struct colvalstruct *);
   tbl[ifile]->colvalue 	= (struct colvalstruct **) 
			  malloc(tbl[ifile]->maxcols 
			* colvalstrc_size);

   if (tbl[ifile]->colvalue ==(struct colvalstruct **) NULL)
   {
       sprintf(tblerror,"Cannot malloc () enough memory.");
       return (TBL_NOMEM);
   }

/********************************
* Initialize Jth column pointer *
* and variables                 *
*********************************/

   for (j=0; j<tbl[ifile]->maxcols; ++j)
   {
       /*---------------------------*/
       /* initialize colifo pointer */
       /* value for Name            */
       /*---------------------------*/

       colstrc_size = sizeof(struct colstruct);
       tbl[ifile]->colinfo[j] 
		 = (struct colstruct *)malloc(colstrc_size);

       if (tbl[ifile]->colinfo[j] == (struct colstruct *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

       /* Initialize the Maxname */
       tbl[ifile]->colinfo[j]->maxname 
			= TBL_MAXHSTR;
       
       colvalstrc_size = sizeof(struct colvalstruct);
       tbl[ifile]->colvalue[j] = (struct colvalstruct *)malloc(colvalstrc_size);
       if (tbl[ifile]->colvalue[j] == (struct colvalstruct *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for Colvalstruct         */
       /*--------------------------------*/
       tbl[ifile]->colvalue[j]->nelem = 0;
       tbl[ifile]->colvalue[j]->cptr = 0; 
       tbl[ifile]->colvalue[j]->iptr  = 0;
       tbl[ifile]->colvalue[j]->dptr  = 0;
       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for Name                 */
       /*--------------------------------*/

       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->name 
			= (char *)malloc(name_size);
       if (tbl[ifile]->colinfo[j]->name == (char *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for datatype             */
       /*--------------------------------*/

       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->dtyp 
			= (char *)malloc(name_size);

       if (tbl[ifile]->colinfo[j]->dtyp == (char *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for unit                 */
       /*--------------------------------*/

       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->unit 
			= (char *)malloc(name_size);

       if (tbl[ifile]->colinfo[j]->unit == (char *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for null                 */
       /*--------------------------------*/

	name_size        = TBL_MAXHSTR * sizeof(char);
	tbl[ifile]->colinfo[j]->null
			 = (char *)malloc(name_size);

	if (tbl[ifile]->colinfo[j]->null == (char *)NULL)
	{
	    sprintf(tblerror,"Cannot malloc () enough memory.");
	    return (TBL_NOMEM);
	}

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for zero                 */
       /*--------------------------------*/

	name_size        = TBL_MAXHSTR * sizeof(char);
	tbl[ifile]->colinfo[j]->zero
			 = (char *)malloc(name_size);

	if (tbl[ifile]->colinfo[j]->zero == (char *)NULL)
	{
	    sprintf(tblerror,"Cannot malloc () enough memory.");
	    return (TBL_NOMEM);
	}

       /*--------------------------------*/
       /* Allocate Memory and Initialize */
       /* value for scale                */
       /*--------------------------------*/

	name_size        = TBL_MAXHSTR * sizeof(char);
	tbl[ifile]->colinfo[j]->scal
			 = (char *)malloc(name_size);

	if (tbl[ifile]->colinfo[j]->scal == (char *)NULL)
	{
	    sprintf(tblerror,"Cannot malloc () enough memory.");
	    return (TBL_NOMEM);
	}

       /*--------------------------------*/	
       /* Initialize Default Name String */
       /*--------------------------------*/

       for (l=0; l<TBL_MAXHSTR; ++l) 
       {
	    tbl[ifile]->colinfo[j]->name[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->dtyp[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->unit[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->null[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->zero[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->scal[l] = '\0'; 
       }

       tbl[ifile]->colinfo[j]->width	 = WIDTH;
       tbl[ifile]->colinfo[j]->endcol	 = ENDCOL;

   } 	/* end of for loop */

   return(TBL_OK);
}
