/****************************************************************************
** Routine Name:                isis_colrallc.c                            **
**                                                                         **
** Description:                 This routine reallocates column memory     **
**                              space if more memory is needed             **
**                                                                         **
** Input Parameters:            ifile   ____ input file number.            **
**                                                                         **
** Output Parameters:           None   					   **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_NOMEM (-2), no memory left	   **
**                                                                         **
** Routines Be Called:          None					   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    (1) Seperate from original prototype code  **
**				    tbl_open.c				   **
**                              (2) Modify code                            **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                      **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


int isis_colrallc(ifile)

int ifile;		/* ifile th input file  */ 

{
   int	 colstrc_size;  /* size of colstruct */
   int   name_size; 	/* size of name defined in colstruct */
   int   j, l;		/* loop index */


/*******************************
* Reallocate Column Info Space *
*******************************/


   /* Reinitialize Number of  Columns */
   tbl[ifile]->maxcols 	+= TBL_MAXCOLS;	

   /* Allocate Memory Size for A colstruct */
   colstrc_size	= sizeof(struct colstruct *)* tbl[ifile]->maxcols;
   tbl[ifile]->colinfo 	= (struct colstruct **) 
			  realloc(tbl[ifile]->colinfo, colstrc_size);

   if (tbl[ifile]->colinfo == (struct colstruct **) NULL)
   {
       sprintf(tblerror, "cannot malloc() enough memory.");
       return (TBL_NOMEM);
   }

   colstrc_size	= sizeof(struct colstruct);
   for (j=tbl[ifile]->maxcols-TBL_MAXCOLS; j<tbl[ifile]->maxcols; ++j)
   {
       /* Allocate Memory and initialize colinfo pointer */
       tbl[ifile]->colinfo[j] 
			= (struct colstruct *)
			   malloc(colstrc_size);
   
       if (tbl[ifile]->colinfo[j] == (struct colstruct *)NULL)
       {
           sprintf(tblerror, "cannot malloc() enough memory.");
           return (TBL_NOMEM);
       }

       /* Initialize the Maxname */
       tbl[ifile]->colinfo[j]->maxname 
			= TBL_MAXHSTR;

       /* Allocate Memory and Initialize Name pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->name 
			= (char *)malloc(name_size);

       /* Allocate Memory and Initialize dtyp pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->dtyp 
			= (char *)malloc(name_size);

       /* Allocate Memory and Initialize unit pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->unit 
			= (char *)malloc(name_size);

       /* Allocate Memory and Initialize null pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->null 
			= (char *)malloc(name_size);

       /* Allocate Memory and Initialize scal pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->scal 
			= (char *)malloc(name_size);

       /* Allocate Memory and Initialize zero pointer  */
       name_size	= TBL_MAXHSTR * sizeof(char);
       tbl[ifile]->colinfo[j]->zero 
			= (char *)malloc(name_size);

       /* Initialize Name String */
       for (l=0; l<TBL_MAXHSTR; ++l) 
       {
	    tbl[ifile]->colinfo[j]->name[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->dtyp[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->unit[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->null[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->scal[l] = '\0'; 
	    tbl[ifile]->colinfo[j]->zero[l] = '\0'; 
       }

       tbl[ifile]->colinfo[j]->width	 = WIDTH;
       tbl[ifile]->colinfo[j]->endcol	 = ENDCOL;
       tbl[ifile]->colinfo[j]->maxname	 = TBL_MAXHSTR;
       tbl[ifile]->colinfo[j]->delcol 	 = 0;

   } 	/* end of j (maxcols) loop */

   return(TBL_OK);
}
