/****************************************************************************
** Routine Name:                isis_column.c                              **
**                                                                         **
** Description:                 This routine checks column name first.     **
**                              It returns a column number if it exists    **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              name    ____ name of the column            **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      stat > 0,              column number       **
**                              stat = TBL_NOCOL (-7)  no such col. exit   **
**                                                                         **
** Routines Be Called:   	None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       Code first written                         **
**   1998-03    Angela Zhang    Modify the code                            **
**   1998-11    John Good       Fixed bug in column name string comparison **
**                              change                                     **
**                                  strncmp(colname, name, strlen(name))   **
**                              to                                         **
**                                  strcmp(colname, name)                  **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


isis_column(ifile, name)

int 	ifile;			/* input file number */	 
char 	*name;			/* pointer to char */	 


/*************************
* Define Local Variables *
**************************/
{
  int	icol;			/* column number */
  int	ipac;			/* index of iapc ascii file */
  char	*colname;		/* column name stored in a structure */
  FILE  *tf;			/* file pointer */

/***************************
* Initialize Local Pointer *
****************************/

  ipac  = tbl_finfo[ifile]->file_handle;
  tf 	= tbl[ipac]->tfile;


/************************
* Check Column name and *
* return the number     *
*************************/

  for (icol=0; icol< tbl[ipac]->ncols; ++icol)	
  {
        colname = tbl[ipac]->colinfo[icol]->name;
	if (strcmp(colname, name) == 0 ) 
        return (icol);		
  }


/*************************
* Give the Error Message *
**************************/

  sprintf(tblerror, "No such column. ");	
  return(TBL_NOCOL);
}

