/****************************************************************************
** Routine Name:                isis_convert.c                             **
**                                                                         **
** Description:                 This routine converts data from a string   **
**			        to  a required data type.  		   **
** 									   ** 
** Input Parameters:            *tbl_ival ____ input data string           **
**                              **ptr     ____ void pointer                **
**                              type      ____ converted data type         **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      stat = TBL_NOTYP ( -9) no such data type   **
**                              stat = TBL_CNVERR(-10) convertion error    **
**                              stat = TBL_ERROR       error               **
**                              stat = TBL_OK          ok                  **
**                                                                         **
** Routines Be Called:          None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    John Good       original written                           **
**   1998-03    Angela Zhang    Modify                                     **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


isis_convert(char *tbl_ival, int type,  void** ptr)

/*************************
* Define Local Variables *
**************************/

{
  char		*endptr;		/* return pointer for converted data */
  double 	dval;			/* converted data */		 	
  long int 	ival;			/* converted data */  	

  char		**cptr;			/* pointer */
  double 	*dptr;			/* pointer */ 
  long int 	*iptr;			/* pointer */ 

  int		cnt;			/* count non digit number */
  int		cnt1;			/* count . */
  int		i;			/* dummy variable */

/********************
* Convert Data Type *
*********************/
 

  switch(type)
  {
      case TBL_DOUBLE:				/* convert string to double */	
        if (*tbl_ival == '\0') return (TBL_CNVERR);
	dptr = (double *) ptr;
	dval = strtod(tbl_ival, &endptr);
        *dptr = dval;
        
        if (endptr - tbl_ival < strlen(tbl_ival)) 
        {
	    for (i = 0; i < strlen(tbl_ival); ++i)
	    {
                if (tbl_ival[i] == 'd'  
                 || tbl_ival[i] == 'D') 
                {
                    tbl_ival[i] = 'e';   
	            dval = strtod(tbl_ival, &endptr);
                    *dptr = dval;
                    if (endptr - tbl_ival < strlen(tbl_ival)) return(TBL_CNVERR);
                    return(TBL_OK);
                }
            }
            return(TBL_CNVERR);
        }
        return(TBL_OK);				
 

      case TBL_INTEGER:				/* convert string to integer */ 
        if (*tbl_ival == '\0') return (TBL_CNVERR);
	iptr = (long int *) ptr;
	ival = strtol(tbl_ival, &endptr, 10);
       *iptr = ival;
        
        if (endptr - tbl_ival < strlen(tbl_ival)) return(TBL_CNVERR);
        return(TBL_OK);				

 
      case TBL_CHARACTER:			/* return char string */
	cptr  = (char **) ptr; 
	cptr  = &tbl_ival;
	*cptr = tbl_ival;
        return(TBL_OK);				
  } 
 
  return(TBL_NOTYP);
}

