/****************************************************************************
**                                                                         **
** Routine Name:                isis_gets.c                                **
**                                                                         **
** Description:                 This routine gets a record line            **
**									   **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Values:      stat > 0, Length of a record field         ** 
**                              stat = TBL_ERROR (-1), read error          **
**                              stat = TBL_NOMEM (-2), no memory left      **
**                              stat = TBL_EOF   (-4), end of file         **
**                              stat = TBL_RDERR (-5), read error          **
**                                                                         **
** Routines Be Called:          None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    Add header and comments			   **
**   2008-11    Iffat Khan      Replace LF+CR by null char   **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/*************************
* Define Including Files *
**************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
int     maxfiles;               /* maximun number of files are allocated */

isis_gets(ifile)
int 	ifile;	 
{
  /*************************
  * Define Local Variables *
  **************************/
  int	i,l, ch;		/* loop index */
  int	stat;			/* status of subroutine */
  FILE  *tf;			/* file point */


  /**********************
  * Check if file exits *
  ***********************/
  if (ifile < 0 || ifile>= maxfiles)
  {
      sprintf(tblerror, "NO such file reference.");
      stat = TBL_ERROR;
      return(stat);
  }

  /**************************
  * Initialize current file *
  ***************************/
  tf = tbl[ifile]->tfile;
  if (tf == (FILE *) NULL) 
  {
      sprintf(tblerror,"File not open");
      stat = TBL_ERROR;
      return(stat);
  }


  /************************
  * Get String and Load a *
  * record into a Record  *
  *************************/

  i = 0;
  while(1)
  {
     ch = getc(tf);
     if (ch == EOF) 
     {
	 if (ferror(tf)) 
	 { 
	     sprintf(tblerror," ERROR on read");
	     stat = TBL_RDERR;
	     return (stat);
         }
	 else
	 {
	     stat = TBL_EOF;
	     return(stat);
	 } 
     }
     if ( (char)ch == '\n')
     {
	tbl[ifile]->rec[i] = '\0';
	++i;
	break;
     }

     tbl[ifile]->rec[i] = (char) ch;
     ++i;

  /**************************
  * Reallocate Memory if it *
  * more memory is Needed   *
  ***************************/

     if (i >= tbl[ifile]->reclen)
     {
         tbl[ifile]->reclen    += TBL_MAXSTR; 
         tbl[ifile]->rec 	= (char *) realloc(tbl[ifile]->rec,
		                  tbl[ifile]->reclen * sizeof(char));

	 for (l = tbl[ifile]->reclen-TBL_MAXHSTR;
	      l < tbl[ifile]->reclen; ++l)
              tbl[ifile]->rec[l]= '\0';
     }   			/* end of if */ 
   }	 			/* end of while loop */


  /* Remove last character if it is LF+CR on WINDOWS */

  if (tbl[ifile]->rec[i-2] == '\r') {
        tbl[ifile]->rec[i-2]='\0';
        --i;
  }  

 
  stat = i-1;
  return(stat);
}
