/****************************************************************************
**                                                                         **
** Routine Name:                isis_keyinfo.c                             **
**                                                                         **
** Description:                 This routine retrieves keyword and value   **
**                              field associated with it.                  **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
**                                                                         **
** Output Parameters:           key	____ defined keyword in ISIS table **
**                                           file.                         **
**				kval    ____ value of a keyword            **
**			        ith     ____ ith keyword                   **
**			        numkey  ____ total number of keywords      **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:          None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    initial written                            **
**                                                                         **
** Version:                     ISIS0.0                                    **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Includeing Files *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/*************************
* define global variable *
**************************/
int         keynum[TBL_MAXFILES];	/* key numbers */


isis_keyinfo(ifile, ith, numkey, key, kval)

int 	ifile;			/* input file number */	 
int 	ith;			/* ith keyword */	 
int 	*numkey;		/* number of keywords in the file*/	 
char 	*key;	         	/* keyword */	 
char 	*kval;	         	/* value field of a keyword */	 
{

    int 	ipac;			/* file number */	 
    /***************
    * assign value *
    ****************/

    ipac = tbl_finfo[ifile]->file_handle;
    *numkey = tbl[ipac]->keynum;

    if (ith+1 > *numkey) return(TBL_ERROR);

    strcpy(key,tbl[ipac]->keyinfo[ith]->key);
    strcpy(kval,tbl[ipac]->keyinfo[ith]->kval);

   
    return(TBL_OK);
}

