/****************************************************************************
** Routine Name:                isis_keymallc.c                            **
**                                                                         **
** Description:                 This routine allocate memory space         **
**                              for keywords.                              **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
** Output Parameters:           none                                       **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success		   **
** 			 	stat = TBL_ERROR (-1), error     	   **
** 			 	stat = TBL_NOEME (-2), no memory	   **
** 			 	stat = TBL_EOF   (-4), end of file	   **
** 			 	stat = TBL_RDERR (-5), read error	   **
**                                                                         **
** Routines Be Called:		none       				   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    initial author				   **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/***************************
* Declear Global Variables *
****************************/
int	num_key;			/* total key number */ 


isis_keymallc(ifile)
int 	ifile;	 			/* file number */


{
   /**************************
   * Declear local variables *
   ***************************/
   int	stat;				/* return status */
   int 	i; 				/* dummy variable */


   /**********************
   * allocate memory for *
   * type, key and kval  *
   ***********************/
   /* allocate the momory */
   tbl[ifile]->keyinfo  = (struct keyword **)
			  malloc(TBL_KEYNUM * sizeof(struct keyword *)); 

   /* not enough memory left */
   if (tbl[ifile]->keyinfo == (struct keyword **) NULL)
       return (TBL_NOMEM);

   /* initialize the pointers */
   num_key = TBL_KEYNUM;
   for (i = 0; i <num_key; ++i)
   {
       tbl[ifile]->keyinfo[i] = (struct keyword *) 
			        malloc(sizeof(struct keyword));
       if (tbl[ifile]->keyinfo[i] == (struct keyword *) NULL)
	   return (TBL_NOMEM);
   }

   tbl[ifile]->keynum = 0;
   return(stat=TBL_OK);
}
