/**************************************************************************** 
** Routine Name:                isis_keyword.c                             **
**                                                                         **
** Description:                 This routine retrieval keywords and        **
**                              value of keywords.                         **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
** Output Parameters:           none                                       **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success		   **
** 			 	stat = TBL_NOEME (-2), no memory	   **
**                                                                         **
** Routines Be Called:		none   i				   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author                             **
**   1998-03    Angela Zhang    (1) Seperate it from original prototype    **
**   				    tbl_open.c    			   **
**                              (2) Modify the code                        **
**   2005-10    Iffat Khan      read value after '=' (not after '= ')      **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/***************************
* Declear Global Variables *
****************************/
int	num_key;			/* dummy variable */
int     keynum[TBL_MAXFILES];		/* keyword number */
char    fix[TBL_MAXFILES];		/* FIX length ISIS_ASCII file */


isis_keyword(ifile )
int 	ifile;	 			/* file number */

{
   /**************************
   * Declear local variables *
   ***************************/
   int	i, j, ival;			/* loop index */
   int	k;				/* string counter */
   int	size;				/* memory size */
   int	num;				/* the number of keyword */
   int	len;				/* length of keyword value */
   int	stat;				/* return status */
   int  nchar;
   char	*rec;				/* pointer to rec */
   char *p;
   FILE *tf;				/* file point */


   /*************************
   * initialize the pointer *
   * and variables          *
   **************************/
   rec 	 = tbl[ifile]->rec;
   num   = tbl[ifile]->keynum;

   tbl[ifile]->keyinfo[num]->ktype = 0; 
   for (i = 0; i < TBL_KEYLEN; ++i)
   {
       tbl[ifile]->keyinfo[num]->key[i]  = '\0';
       tbl[ifile]->keyinfo[num]->kval[i] = '\0';
   }
   for (i = 0; i < TBL_COMMENT;  ++i)
       tbl[ifile]->keyinfo[num]->kcomment[i]  = '\0';


   /***************
   * check memory * 
   ****************/
   if (num >= TBL_KEYNUM )
   {
       num_key		    = TBL_KEYNUM + num;
       size 		    = num_key * sizeof (struct keyword *);
       tbl[ifile]->keyinfo  = (struct keyword **)
			      realloc(tbl[ifile]->keyinfo, size); 

       if (tbl[ifile]->keyinfo == (struct keyword **)NULL)
           return (TBL_NOMEM);

       for (i=num_key-TBL_KEYNUM; i<num_key; ++i)
       {
           size = sizeof (struct keyword);
           tbl[ifile]->keyinfo[i] = (struct keyword *) malloc(size);
           if (tbl[ifile]->keyinfo[i] == (struct keyword *) NULL)
	       return (TBL_NOMEM);
       }
   }


   /**************
   * get keyword *
   ***************/
   j = 0;
   if (rec[1] == ' ' || rec[1] == '\0') 
   {
       return(4);			/* comment line */
   }
   else
   {
       i = 1;
       tbl[ifile]->keynum = tbl[ifile]->keynum + 1;
       while (i < TBL_KEYLEN && rec[i] != '=')
       {
	   if (rec[i] != '"' ) 
	   {
	       tbl[ifile]->keyinfo[num]->key[j] = rec[i];
	       j = j + 1;
           }
	   i = i + 1;
       }

       /* get rid of the blanks */
       for (k = j-1; k > 0; --k)
       {
	   if (tbl[ifile]->keyinfo[num]->key[k] != ' ')
               break;
	   else
	       tbl[ifile]->keyinfo[num]->key[k] = '\0';
       }

       /* get rid of char, int, real */
	if (strncmp(tbl[ifile]->keyinfo[num]->key,"char",4) == 0
	 || strncmp(tbl[ifile]->keyinfo[num]->key,"CHAR",4) == 0)
	{
           nchar = 5;
	   if (strncmp(tbl[ifile]->keyinfo[num]->key,"character",9) == 0
	    || strncmp(tbl[ifile]->keyinfo[num]->key,"CHARACTER",9) == 0)
           nchar = 10; 
	    strcpy(tbl[ifile]->keyinfo[num]->key,
	                            tbl[ifile]->keyinfo[num]->key+nchar);
            tbl[ifile]->keyinfo[num]->ktype = TBL_CHARACTER;
	}

	if (strncmp(tbl[ifile]->keyinfo[num]->key,"int",3) == 0 ||
	    strncmp(tbl[ifile]->keyinfo[num]->key,"INT",3) == 0)
	{
	    strcpy(tbl[ifile]->keyinfo[num]->key,
	                            tbl[ifile]->keyinfo[num]->key+3);
            tbl[ifile]->keyinfo[num]->ktype = TBL_INTEGER;
	}

	if (strncmp(tbl[ifile]->keyinfo[num]->key,"float",5) == 0 ||
	    strncmp(tbl[ifile]->keyinfo[num]->key,"FLOAT",5) == 0)
	{
	    strcpy(tbl[ifile]->keyinfo[num]->key,
	                            tbl[ifile]->keyinfo[num]->key+5);
            tbl[ifile]->keyinfo[num]->ktype = TBL_DOUBLE;
	}

	if (strncmp(tbl[ifile]->keyinfo[num]->key,"real",4) == 0 ||
	    strncmp(tbl[ifile]->keyinfo[num]->key,"REAL",4) == 0)
	{
	    strcpy(tbl[ifile]->keyinfo[num]->key,
	                            tbl[ifile]->keyinfo[num]->key+4);
            tbl[ifile]->keyinfo[num]->ktype = TBL_DOUBLE;
	}

        p = tbl[ifile]->keyinfo[num]->key;
        skipwhite(p);
        strcpy(tbl[ifile]->keyinfo[num]->key, p);
       /* excluding starting blanks in a value part of a character string */
       i = i + 1;
/* skip blanks */
       while (i < TBL_KEYLEN && rec[i] == ' ') 	
	   i = i + 1;   

       /* excluding " and ' in a character string */
       if (rec[i] == '"' || rec[i] == '\'')
	   i = i + 1;

       /* find the value of a keyword  */
       j = 0;
/*when we are on windows we need to not end with \ since that can 
 *be a windows file name */
       while (i < TBL_KEYLEN && 
	      rec[i] != '\0' && 
#ifndef SIRTF_WIN
              rec[i] != '\\' && 
#endif
              rec[i] != '"' 
              && rec[i] != '\'')
       {
	   tbl[ifile]->keyinfo[num]->kval[j] = rec[i];
	   i = i + 1;
	   j = j + 1;
       }
       tbl[ifile]->keyinfo[num]->kval[j]= '\0';
       ival = i;
       if (rec[i] == '"' || rec[i] == '\'')
        ival++;

       len = strlen(tbl[ifile]->keyinfo[num]->kval);
       for (i=len-1; i>0; --i)
       {
	   if (tbl[ifile]->keyinfo[num]->kval[i] == ' ')
	       tbl[ifile]->keyinfo[num]->kval[i] = '\0';
           else
	       break;
       }
       p = tbl[ifile]->keyinfo[num]->kval;
       skipwhite(p);
       strcpy(tbl[ifile]->keyinfo[num]->kval, p);
       if (tbl[ifile]->keyinfo[num]->ktype == TBL_DOUBLE ||
           tbl[ifile]->keyinfo[num]->ktype == TBL_INTEGER) 
       {
         p = strrchr(tbl[ifile]->keyinfo[num]->kval,'/');
         if ( p != NULL)
         {
            strcpy(tbl[ifile]->keyinfo[num]->kcomment, p);
            for (i=0; i<len-1; i++)
            {
	      if (tbl[ifile]->keyinfo[num]->kval[i] == ' '
                  || tbl[ifile]->keyinfo[num]->kval[i] == '/')
                  break;
            }
            tbl[ifile]->keyinfo[num]->kval[i] = '\0';
         }
       }
       else if (tbl[ifile]->keyinfo[num]->ktype == TBL_CHARACTER)
       {
         if (strncasecmp(tbl[ifile]->keyinfo[num]->key,"comment",7) == 0
             || strncasecmp(tbl[ifile]->keyinfo[num]->key,"history",7)  == 0)
         {
           if (strlen(tbl[ifile]->keyinfo[num]->kval) == 0)
           {
	     strcpy(tbl[ifile]->keyinfo[num]->kval,
	                            tbl[ifile]->keyinfo[num]->key+7);
             tbl[ifile]->keyinfo[num]->key[7] = '\0';
             
           }
         }
         else
         {
            j = 0;
            i = ival;
            while (i < TBL_KEYLEN && rec[i] != '\0') 
            {
	     tbl[ifile]->keyinfo[num]->kcomment[j] = rec[i];
	     i = i + 1;
	     j = j + 1;
            }
            tbl[ifile]->keyinfo[num]->kcomment[j]= '\0';
        } 
    }
  }

   /* ------------------------------ */
   /* IPAC FIXED length table format */ 
   /* ------------------------------ */
   if (strcmp("fixlen",tbl[ifile]->keyinfo[num]->key) == 0)
   {
       for (i = 0; i < TBL_KEYLEN; ++i)
       {
	    if (tbl[ifile]->keyinfo[num]->kval[i] != ' ') 
            {         
	       fix[ifile] = tbl[ifile]->keyinfo[num]->kval[i];
	       break;
            }
       }
   } 

  
   return(TBL_OK);
}
