/****************************************************************************
**                                                                         **
** Routine Name:                isis_mallc.c                               **
**                                                                         **
** Description:                 This routine allocates initial memory      **
**                              space. It will dynamiclly allocate         **
**                              memory space on request basis.             **
**                                                                         **
** Input Parameters:            ifile   ____ indicate the current file     **
**					     number.			   **
**                                                                         **
** Output Parameters:           none   					   **
**                                                                         **
** Function Return Values:      stat > 0 (file number), success 	   **
**                              stat = TBL_NOMEM (-2),  no memory	   **
**									   **
** Routines Be Called:          None					   **
**									   **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       Code first written                         **
**   1998-03    Angela Zhang    Modify prototype code and seperate         **
**                              code into several units.                   **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* Define include files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
int     maxfiles;       /* maximun number of files are allocated */


int isis_mallc(ifile)
int *ifile;

{
   /*************************
   * Define Local Variables *
   **************************/
   int	 tblstrc_size;  /* size of tblstruct */
   int   header_size;	/* size of header information */
   int   rec_size; 	/* size of rec defined in tblstruct */
   int   i, j, l;	/* loop index */
   int   stat;		/* status */


   /************************************
   * Allocate Initial Table Info Space *
   *************************************/

   if (maxfiles == 0) /* only once */
   {
       maxfiles		= TBL_MAXFILES;

       /* Allocate Default Memory Size for A Tblstruct */
       tblstrc_size	= sizeof(struct tblstruct *);
       tbl 		= (struct tblstruct **) 
			  malloc(TBL_MAXFILES 
			* tblstrc_size);
   
       if (tbl == (struct tblstruct **)NULL)
       {
           sprintf(tblerror, "cannot malloc() enough memory.");
           return(TBL_NOMEM);
       }

       for (i=0; i<maxfiles; ++i)	 
       {

         /* Allocate Default Memory for maxfiles */
         tblstrc_size	= sizeof(struct tblstruct);
         *(tbl+i) 	= (struct tblstruct *) 
			   malloc(tblstrc_size);
         if (*(tbl+i) == (struct tblstruct *)NULL)
         {
             sprintf(tblerror, "cannot malloc() enough memory.");
             return(TBL_NOMEM);
         }
         tbl[i]->tfile 	= (FILE *)NULL;
       }
   } /* end of if (maxfiles) */

   /********************
   * Find Unused Table *
   *********************/
   for (i=0; i<maxfiles; ++i)
   {
	if (tbl[i]->tfile == (FILE *)NULL) break;
   }


   /**************************
   * Allocate Table Momory   *
   * Space If More Is Needed *
   ***************************/
   if (i<maxfiles)
       *ifile = i;
   else
   {
       maxfiles += TBL_MAXFILES;
       tbl	= (struct tblstruct **)
		   realloc(tbl,maxfiles 
		*  sizeof(struct tblstruct *));
       if (tbl  == (struct tblstruct **) NULL)
       {
           sprintf(tblerror, "cannot malloc() enough memory.");
           return(TBL_NOMEM);
       }
       for (j=maxfiles - TBL_MAXFILES; j<maxfiles; ++j)
       {
	 *(tbl+j) = (struct tblstruct *)
		     malloc(sizeof (struct tblstruct));
         if (*(tbl+j)  == (struct tblstruct *) NULL)
         {
             sprintf(tblerror, "cannot malloc() enough memory.");
             return(TBL_NOMEM);
         }			/* end of if (tbl+j)  */
		     
	 tbl[j]->tfile = (FILE *) NULL;
       } 			/* end of for loop */
       *ifile = maxfiles - TBL_MAXFILES;
   } 				/* end of if i */
  
   tbl[*ifile]->delrow	= 0; 		
   /*************************************
   * Allocate Initial Column Info Space *
   **************************************/
   /* Allocate Default Rec Memory */
   rec_size		= TBL_MAXSTR * sizeof(char); 
   tbl[*ifile]->rec	= (char *)malloc(rec_size);

   /* Initialize Default Rec */
   for (j=0; j<TBL_MAXSTR; ++j) 
	  tbl[*ifile]->rec[j]= '\0'; 

   /* Initialize Default Number of Headers */
   tbl[*ifile]->header	= HEADER; 		

   /* Initialize Default Record Length */
   tbl[*ifile]->reclen = TBL_MAXSTR;	

   /* Allocate Column Memory Size */
   stat 		= isis_colmallc(*ifile);
   if (stat <0) return(stat); 

   return(*ifile);	/* return file number, if ok */

}
