/****************************************************************************
**								           **	
** Routine Name:		isis_open.c				   **
**								           **	
** Description:			This routine opens number of files. 	   **
**				It dynamicaly allocates memory space       **
**			        when more is needed.       		   ** 
**									   **
** Input Parameters:		fname 	____ input file name pointer.	   ** 
**									   **
** Output Parameters:  		numcol	---- number of column              **
**									   **
** Function Return Values:	stat = file number,    suceses             **
**				stat = TBL_NOMEM (-2), no memory left      **
**									   **
** Routine Be Called:		isis_mallc.c			 	   **
** 		 		isis_header.c			 	   **
** 		 		isis_submallc.c			 	   **
** 		 		isis_valmallc.c			 	   **
**									   **
** Revision History:							   **
**								           **	
**   19??-??	John Good	initial author				   **
**   1998-03    Angela Zhang    (1) Modify and seperate code from its      **
**			            original tbl_open.c                    **
**				(2) Add memory allocation for subtitles	   **
**			        (3) Add status checking.		   **
**   2004-03   Iffat Khan       Modified for table write                   **
**   2005-10   Iffat Khan       Modified for temp file while appending     **
**                              data in input files.                       **
**									   **
** Version:			TBL0.0					   **
**									   **
**									   **
** Infrared Science Information System					   **
** California Institute of Technology					   **
**									   **
** Copyright (c) 1998, California Institute of Technology.		   **
** U. S. Government Sponsorship under NASA Contract			   **
** NAS7-???? is acknowledged.						   **
**									   **
*****************************************************************************/

/***********************
* Define include files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Declear global variables *
***************************/
long int  key_bytes[TBL_MAXFILES];      /* total keyword bytes */
long int  rec_bytes[TBL_MAXFILES];      /* total bytes in a record */
char      fix[TBL_MAXFILES];            /* fixed ISIS_ASCII file */
char      mode;
int       tmp_output[TBL_MAXFILES];


isis_open(fname, numcol)

char *fname;			/* input filename */
int  *numcol;			/* number of column in a file */


/************************
* Define Local Variable *
*************************/

{
  int   i, j, k, l;		/* loop index */
  int	stat;			/* return status */
  int	ifile;			/* input file number */
  int	ofile;			/* output file number */
  char  *rec;			/* current record */
  FILE  *tf;			/* current file */
  int	isis_mallc();


/*******************************
* Initial Allocation of Memory *
********************************/



/*************
* Open files *
**************/
  if (mode == 'r') {
    stat = isis_mallc(&ifile);
    if (stat < 0) return(stat); 
    tbl[ifile]->tfile = fopen(fname, "r");
    if (tbl[ifile]->tfile == NULL) {
      printf("Error in opening input file \n");
      return(TBL_ERROR);
    }
  }
  else if (mode == 'w') {
    stat = isis_mallc(&ofile);
    if (stat < 0) return(stat); 
    tmp_output[ofile] = 0;
    if (strlen(fname) == 0 ||
     (strlen(fname) == 1 && fname[0] == ' ') )
    {
      tbl[ofile]->tfile = tmpfile();         
      tmp_output[ofile] = 1;
    } 
    else
      tbl[ofile]->tfile = fopen(fname, "w");
    if (tbl[ofile]->tfile == NULL) {

      printf("Error in opening output file \n");
      return(TBL_ERROR);
    

   }  
     return (ofile);       /*return here for output files */
  }

/******************************
* Allocate Memory for Keyword *
*******************************/

  stat   = isis_keymallc(ifile);
  if (stat < 0) return(stat); 


/*************************
* Get Header Information *
**************************/

  stat   = isis_header(ifile, numcol); 
  if (stat < 0) return(stat); 


/********************************
* Allocate Memory for Subtitles *
*********************************/

  stat   = isis_submallc(ifile, *numcol);    
  if (stat < 0) return(stat); 
 
  
/****************************
* Allocate Memory for Value *
*****************************/

  stat   = isis_valmallc(ifile);  
  if (stat < 0) return(stat); 

/***************************
* get subtitle information *
****************************/

  stat = isis_subtit(ifile);

  return(ifile);
}
