/***************************************************************************e
**                                                                         **
** Routine Name:                isis_read.c                                **
**                                                                         **
** Description:                 This routine reads a required data record. **
**				If row number of a required record is less **
**				than curent data record, it rewinds. It	   **
**				calls isis_subtit.c to save all of the     **
**				subtitle information.	                   **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              erow    ____ row number want to read       **
**                                                                         **
** Output Parameters:           None   				 	   **
**                                                                         **
** Function Return Values:      stat = TBL_OK     (0), success             **
**                              stat = TBL_NOMEM (-2), no memory left      **
**                              stat = TBL_EOF   (-4), end of file         **
**                              stat = TBL_RDERR (-5), read error          **
**                                                                         **
** Routines Be Called:							   **
**									   **
**				isis_subtit.c				   **
**				isis_gets.c			           **
**									   **
**				isis_rewind.c				   **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initial author				   **
**   1998-03    Angela Zhang    (1) Modify code to read number of records  ** 
**                              (2) Add rewind if needed                   **
**                                                                         **
**   2008-11    I. Khan         Modify code to replace CR on WINDOWS       ** 
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
long int	 srow;			/* initial row number */


isis_read(ifile, erow)
int 	ifile;				/* input file number */	 
int	erow;


{
  /*************************
  * Define Local Variables *
  **************************/
  int		i, j, k, ii, ipac;	/* loop index */
  int		irow, icol;		/* loop index */
  int		isubtit;		/* subtitle row index */
  int		colb,cole,kk;		/* local index */
  int		stat;			/* status of subroutine */
  char		*rec;			/* recorder pointer */
  FILE  	*tf;			/* file pointer */


  /****************************
  * Initialize Local Pointers *
  *****************************/
  ipac  = tbl_finfo[ifile]->file_handle;
  tf 	= tbl[ipac]->tfile;
  rec 	= tbl[ipac]->rec;
  stat  = 0;

  /*********************
  * Read a String Line *
  **********************/
  if (srow == 0)
  {
      if (tbl[ipac]->header == 1)	/* already find a vertical bar */
      {
	   while (rec[0] == '|' || rec[0] == '\\' || rec[0] =='-')
           {
	       stat = isis_gets(ifile);
               if (stat < 0) return(stat);
           }
      } /* end if (header) */
      else				/* never find a vertical bar */
      {
          tbl[ipac]->header = 1;	/* as if a vertical bar is found */
          while (1)
          {
	      if (rec[0] != '|'  && rec[0] != '\\' && 
	          rec[0] != '-'  ) break;
	      stat = isis_gets(ifile);
	      if (stat <0) return(stat);
          }  
      }
      if (stat < 0) return(stat);
  }
  else
  {

      /*##########################
       # Skip number of lines to #
       # get required record     #
       ##########################*/ 
      for (irow=srow-1; irow<erow; ++irow)
      {    
          stat = isis_gets(ifile); 
	  if (stat <0) return(stat);
          while (rec[0] == '\\' || rec[0] == '-' || rec[0] == '|')
          {
	      stat = isis_gets(ifile); 
              if (stat < 0) return (stat);
          }
      }

     /*##################################
      # check to see if it needs rewind # 
      ##################################*/
      if (erow < srow-1) 
      {
	  stat=isis_rewind(ifile);
          if (stat <0) return;
      
	  for (irow=srow; irow<erow; ++irow)
          {    
              stat = isis_gets(ifile);
	      if (stat <0) return(stat);
              while (rec[0] == '\\' || rec[0] == '-' || rec[0] == '|')
                  stat = isis_gets(ifile); 
              if (stat < 0) return (stat);
	  }
      }
      if (rec[strlen(rec)]  == '\n')
          rec[strlen(rec)]  = '\0';

/* extra check for LF+CR on WINDOWS  */
      if (rec[strlen(rec)]  == '\r')
          rec[strlen(rec)]  = '\0';
  }
  srow = erow + 1;
  return(TBL_OK);
}
