/****************************************************************************
**                                                                         **
** Routine Name:                isis_rewind.c                              **
**                                                                         **
** Description:                 This routine rewinds a table file          **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Value:       stat = TBL_OK    ( 0), sucses              **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:							   **
**                              isis_gets.c				   **
** Revision History:                                                       **
**                                                                         **
**   19??-??    John Good       initila author				   **
**   1998-03    Angela Zhang    Add header and comment                     **
**                                                                         **
** Version:                     0.0                                        **
**                                                                         **
**                                                                         **
** Infraed Science Information System                                      **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* define global variables *
***************************/
long int	srow;		/* initial row number */
int     	maxfiles;       /* maximun number of files are allocated */


isis_rewind(ifile)
int ifile;

{
   /*************************
   * define local variables *
   **************************/
   int   i, stat, ipac;
   FILE *tf;
   char *rec;

   /************************
   * initialize  variables *
   *************************/
   ipac    = tbl_finfo[ifile]->file_handle;
   srow    = 0; /* reset row number */

   /************************
   * check file references *
   *************************/
   if (ipac < 0 || ipac >= maxfiles)
   {
       sprintf(tblerror, "No such file reference.");
       return(TBL_ERROR);
   }

   tf = tbl[ipac]->tfile;
   if (tf == (FILE *)NULL)
   {
       sprintf(tblerror, "File not open.");
       return(TBL_ERROR);
   }

   /******************
   * rewind the file *
   *******************/
   rewind(tf);
   rec = tbl[ipac]->rec;
   while(1)
   {
       stat = isis_gets(ifile);
       if (stat < 0) return(stat);
       if(rec[0] != '\\' && rec[0] != '|' && rec[0] != '-') break;  
   }

   tbl[ipac]->header = 0;
   return(stat);
}
