/****************************************************************************
** Routine Name:                isis_scaling.c                             **
**                                                                         **
** Description:                 This routine scales a data value           **
**                                                                         **
** Input Parameters:            *scale ____ scaling factor                 **
**				*offset____ data offset                    **
**                              *inptr ____ input pointer                  **
**                              type   ____ data type                      **
**                                                                         **
** Output Parameters:                                                      **
**                              *outptr____ output pointer                 **
**                                                                         **
** Function Return Values:      stat = TBL_NOTYP ( -9) no such data type   **
**                              stat = TBL_CNVERR(-10) convertion error    **
**                              stat = TBL_ERROR ( -1)  error              **
**                              stat = TBL_WARNING( 4) can't be scaled     **
**                              stat = TBL_OK     ( 0) ok                  **
**                                                                         **
** Routines Be Called:          None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Anzhen Zhang    Original written                           **
**                                                                         **
** Version:                     0.0                                        **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


isis_scaling(type, scale, offset, inptr, outptr)

int	type;
void    *scale;
void    *offset;
void    *inptr;
void    **outptr;


/*************************
* Define Local Variables *
**************************/
{
  double 	*tdptr;			 
  double 	*sd;			 
  double 	*od;			 

  long int 	*tiptr;			 
  long int 	*si;			 
  long int 	*oi;			

  char		**cptr;



  switch(type)
  {
      case TBL_DOUBLE:
	tdptr  = (double *) outptr;
	sd     = (double *) scale;
	od     = (double *) offset;
	if (*sd == 0.0)
	{
	    *tdptr = 0.0;
	    return(TBL_WSCAL);
        }
	*tdptr = *(double *)inptr * (*sd) + (*od);
        return(TBL_OK);
 
      case TBL_INTEGER:
	tiptr  = (long int *) outptr;
	si     = (long int *) scale;
	oi     = (long int *) offset;
	if (*si == 0)
	{
	    *tiptr = 0;
	    return(TBL_WSCAL);
        }
        *tiptr = *(long int *)inptr * (*si) + (*oi);
        return(TBL_OK);
 
      case TBL_CHARACTER:
        cptr   = (char **)outptr; 
        *cptr  = *(char **)inptr; 

        return(TBL_WARNING);
  } 
 
  return(TBL_ERROR);
  }

