/****************************************************************************
**                                                                         **
** Routine Name:                isis_submallc.c                            **
**                                                                         **
** Description:                 This routine allocates subtitle memory     **
**			        space.					   ** 
**                                                                         **
** Input Parameters:            ifile   ____ file number                   **
**			        ncol    ____ number of column              ** 
**									   **
** Output Parameters:           None   					   **
**                                                                         **
** Function Return Values:      TBL_OK    ( 0), success                    **
**                              TBL_NOMEM (-2), No enough memory           **
**                                                                         **
** Routine Be Called:		None                                       **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    Initial written  			   **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/***************************
* Declear global variables *
****************************/
char 		***tbl_subtit;  /* contains header */
int	        maxcols;	/* the maxnum of fields */
long int	srow;	        /* initial row number */


isis_submallc (ifile, ncols)
int 	ifile; 			/* input file number */
int 	ncols; 			/* number of column */

{
   /**************************
   * Declear local variables *
   ***************************/
   int	i, k;			/* string counter for a column */
   int	icol;			/* column loop index */
   int	irow;			/* row loop index */
   int	col_len;		/* width of a column */


   /***********************
   * initialize variables *
   ************************/
   srow       = 0;


   /******************************
   * Alllocate Memory for Subtit *
   *******************************/
   if (tbl_subtit == (char ***) NULL)
   {
       tbl_subtit = (char ***) 
                malloc(TBL_MAXSUBTIT * sizeof(char **)); 

       if (tbl_subtit == (char ***) NULL)
       {
           sprintf(tblerror, "Cannot malloc() enough memory. ");
           return(TBL_NOMEM);
       }


       /***************************
       * Initialize array printer *
       ****************************/
       for (irow=0; irow<TBL_MAXSUBTIT; ++irow)
       {
           tbl_subtit[irow] = (char **)
			      malloc(TBL_MAXCOLS * sizeof(char *));	

           if (tbl_subtit[irow] == (char **) NULL)	
           {
               sprintf(tblerror, "Cannot malloc() enough memory. ");
               return(TBL_NOMEM);
           }

           for (icol=0; icol<TBL_MAXCOLS; ++icol)
           {
               tbl_subtit[irow][icol] = (char *)
					malloc(sizeof(char) * TBL_MAXSUBLEN);	
               if (tbl_subtit[irow][icol] == (char *) NULL)
               {
                   sprintf(tblerror, "Cannot malloc() enough memory. ");
                   return(TBL_NOMEM);
               }
	       for (k = 0; k < TBL_MAXSUBLEN; ++k)
	           tbl_subtit[irow][icol][k] = '\0'; 
           } /* end of icol loop */
       }     /* end of irow loop */
       maxcols = TBL_MAXCOLS;
   }

   if (ncols> maxcols)
   {
       /***************************
       * ReInitialize array printer *
       ****************************/
       maxcols    = maxcols + TBL_MAXCOLS;
       for (irow=0; irow<TBL_MAXSUBTIT; ++irow)
       {
           tbl_subtit[irow] = (char **)
		               realloc(tbl_subtit[irow],
			       maxcols * sizeof(char *));	

           if (tbl_subtit[irow] == (char **) NULL)	
           {
               sprintf(tblerror, "Cannot malloc() enough memory. ");
               return(TBL_NOMEM);
           }

           for (icol = maxcols-TBL_MAXCOLS; icol < maxcols; ++icol)
           {
               tbl_subtit[irow][icol] = (char *)
					malloc(sizeof(char) * TBL_MAXSUBLEN);	
               if (tbl_subtit[irow][icol] == (char *) NULL)
               {
                   sprintf(tblerror, "Cannot malloc() enough memory. ");
                   return(TBL_NOMEM);
               }
	       for (k=0; k<TBL_MAXSUBLEN; ++k)
	           tbl_subtit[irow][icol][k] = '\0'; 
           } /* end of icol loop */
       } /* end of irow loop */
   }
   return(TBL_OK);
}
