/****************************************************************************
**                                                                         **
** Routine Name:                isis_subtit.c                              **
**                                                                         **
** Description:                 This routine assign subtitle values for    **
**			        current row.        		           **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              irow    ____ current row number            **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Value:       stat = TBL_OK (0)                          **
**                                                                         **
** Routines Be Called:		None			                   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    Initial written                            **
**                                                                         **
** Version:                     TBL0.0                                    **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Declear Global Variables *
***************************/
/*char tbl_subtit[TBL_MAXSUBTIT][TBL_MAXCOLS][TBL_MAXSUBLEN]; */	/* ISIS header */
char 		***tbl_subtit;	/* ISIS header */
long int	key_bytes[TBL_MAXFILES] ;/* total keyword bytes */


isis_subtit(ifile)
int 	ifile; 			/* input file number */

{
   /**************************
   * Declear local variables *
   ***************************/
   int	k;			/* string counter for a column */
   int	icol;			/* column index */
   int	ncols;			/* number of columns */
   int	i,j;			/* dummy variable */
   int  bgncol;			/* column starting position */
   int  endcol;			/* column ending position */
   int  num_tit;		/* dummy variable */


   /************************
   * Assign Subtitle Value *
   *************************/
   ncols = tbl[ifile]->ncols;

   i = 0;
   while ((tbl[ifile]->rec[0] == '|' || tbl[ifile]->rec[0] == '-')
	  && i < TBL_MAXSUBTIT)
   {
       for (icol=0; icol<ncols; ++icol)
       {
	    if (icol == 0)
		bgncol = 0;
            else
		bgncol = tbl[ifile]->colinfo[icol-1]->endcol;

	    endcol = tbl[ifile]->colinfo[icol]->endcol; 

	    j = 0;
	    for (k = bgncol; k <= endcol; ++k)
	    {
	         tbl_subtit[i][icol][j] = tbl[ifile]->rec[k];

		 if (i == 0)
		 {
	             if (tbl[ifile]->rec[k] != '|' && 
/*		         tbl[ifile]->rec[k] != ' ' &&  */
		         tbl[ifile]->rec[k] != '-') j = j + 1;
                 }
		 else
		 {
/*	             if (tbl[ifile]->rec[k] != '|' && 
		         tbl[ifile]->rec[k] != ' ')	 */

	             if (tbl[ifile]->rec[k] != '|') 
		         j = j + 1;
                 }

            }
	    tbl_subtit[i][icol][j] = '\0';
	    for (j=strlen(tbl_subtit[i][icol])-1; j>0; --j)
	    {
		 if (tbl_subtit[i][icol][j] != ' ') break;
		 tbl_subtit[i][icol][j] = '\0';
            }
	    for (j=0; j<strlen(tbl_subtit[i][icol]); ++j)
	    {
		 if (tbl_subtit[i][icol][j] != ' ')
		 {
		     strcpy(tbl_subtit[i][icol],tbl_subtit[i][icol]+j);
		     break;
                 }
            }
       }
       isis_gets(ifile);
       ++i;
       num_tit	= i;
   }

   /*********************
   * Total Bytes Before * 
   * Data Starts        *
   *********************/
   key_bytes[ifile] = key_bytes[ifile] + num_tit * (endcol + 2);

   for (i=0; i<ncols; ++i)
   {
       strcpy(tbl[ifile]->colinfo[i]->dtyp, tbl_subtit[1][i]);
       strcpy(tbl[ifile]->colinfo[i]->unit, tbl_subtit[2][i]);
       strcpy(tbl[ifile]->colinfo[i]->null, tbl_subtit[3][i]);
       strcpy(tbl[ifile]->colinfo[i]->scal, tbl_subtit[4][i]);
       strcpy(tbl[ifile]->colinfo[i]->zero, tbl_subtit[5][i]);
   }

   return(0);
}
