/****************************************************************************
** Routine Name:                isis_type.c                                **
**                                                                         **
** Description:                 This routine returns data type             **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              icol    ____ column number                 **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
** Function Return Value:       stat = 1,              double              **
**                              stat = 2,              integer             **
**                              stat = 3,              character           **
**                              stat = TBL_NOTYP (-9), no such data type   **
**                              stat = TBL_NOCOL (-10), no such data type   **
**                                                                         **
** Routines Be Called:   	None                                       **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    original written                           **
**                                                                         **
** Version:                     0.0                                        **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                      **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


isis_type(ifile, icol)

int 	ifile;			/* file number */	 
int 	icol;			/* column number*/	 



/*************************
* Define Local Variables *
**************************/
{
  int   stat;                   /* data type */
  int   ipac;                   /* ifile number */


  ipac = tbl_finfo[ifile]->file_handle;
  stat = TBL_OK;
/**********************
* Check Column number *
***********************/

  if (icol<0 || icol> tbl[ipac]->ncols)	
      return (TBL_NOCOL);		


/**********************
* Determine Data Type *
***********************/

  switch (tbl[ipac]->colinfo[icol]->dtyp[0])
  {
     case 'd':
       stat = TBL_DOUBLE;
       return(stat);

     case 'r':
       stat = TBL_DOUBLE;
       return(stat);

     case 'f':
       stat = TBL_DOUBLE;
       return(stat);

     case 'i':
       stat = TBL_INTEGER;
       return(stat);

     case 'c':
       stat = TBL_CHARACTER;
       return(stat);
  }

  return(TBL_NOTYP);
}

