/****************************************************************************
**                                                                         **
** Routine Name:                tbl_close.c                                **
**                                                                         **
**                                                                         **
** Description:                 This routine closes a table file and       **
**                              free occupied memory.                      **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                                                                         **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK (0),     sucess              **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
**                                                                         **
** Usage:			tbl_close(int ifile)                       **
**                                                                         **
**                                                                         **
** Routines Be Called:   	FITS library calls:                        **
** 				     ffclos.c				   **
**                                                                         **
**  				ISIS library calls:                        **
** 				     isis_close.c			   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    first written                              **
**   2004-08    Iffat Khan      modified to append in input file           **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* define Include Files *
************************/

/* this affects include file tbl.h to have it initialize the globals and allocate storage */
#define  tbl_define_globals

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
double      *tscal[TBL_MAXFILES];	/* scale factor */ 
double      *tzero[TBL_MAXFILES];	/* data offset */

long int    tbcol[TBL_MAXFILES][200];	/* ASCII table ending col. byte num. */ 
int	    tabletype;		/* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */ 
int         keynum[TBL_MAXFILES];	/* key number */
int         tmp_output[TBL_MAXFILES];	/* option to indicate temp file output */

char        **ttype[TBL_MAXFILES];	/* data type */
char        **tform[TBL_MAXFILES];	/* data format */
char        **tunit[TBL_MAXFILES];	/* data unit */
char        **tdisp[TBL_MAXFILES];	/* display format */



tbl_close(ifile)
int 	ifile;			/* input file number */	 


{
    /*************************
    * Define Local Variables *
    **************************/
    int	stat;		/* status */
    int	file_no;	/* file index */
    int link_no;        /* output file linked? */


    /*****************
    * get table type *
    ******************/
    file_no    = tbl_finfo[ifile]->file_handle;
    tabletype  = tbl_finfo[ifile]->table_type;
    link_no   = tbl_finfo[ifile]->link_handle;

    /************************
    * close ISIS ASIIC file *
    *************************/
    if (tabletype == ISIS_ASCII)
	stat = isis_close(file_no);

    tabletype = 0;
    tbl_finfo[ifile]->fileptr = (FILE *)NULL;

  return(stat);
}

