/****************************************************************************
**                                                                         **
** Routine Name:                tbl_colinfo.c                              **
**                                                                         **
** Description:                 This routine retrieves a column information**
**                                                                         **
** Input Parameters:            					   **
** 				ifile   ____ input file number             **
**                              icol    ____ column number                 **
**                                                                         **
** Output Parameters:                                                      **
**                              *cinfo  ____ Column information            **
**					     structure  		   **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:          FITS library calls:			   **
**				     ffgacl.c				   **
**				     ffgbcl.c				   **
**									   **
**                              ISIS library calls:                        **
**				     none        		           **
**									   **
** Revision History:                                                       **
**									   **
**   1998-03    Angela Zhang    first written	                           **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Includeing Files *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
int     tabletype;              /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */


tbl_colinfo(ifile, icol, cinfo)

int 		   ifile;	/* input file number */	 
int 		   icol;	/* column index */	 
struct tbl_colinfo *cinfo;	/* struct for column information */	 

{
    int 	stat;		/* column position index */	 
    int 	i,j;		/* dummy variable */	 
    int 	len;		/* len of the variable */	 
    int 	file_no;	/* file index */	 
    long int 	tbcol;		/* column position index */	 
    double	scale;		/* scaling factor */
    double	offset;	        /* offset */
    char 	namet[80];	/* column name */	 
    char 	dtypt[80];	/* datatype in character string */ 
    char 	unitt[80];	/* unit in character string */	 
    char 	nullt[80];	/* null value in character string */ 
    char 	scalt[80];	/* scaling in character string */
    char 	zerot[80];	/* offset in character string */
    char 	dispt[80];	/* display format */	 
    char 	tmp[80];		 


    /************************
    * allocate memory space *
    * for tbl_finfo         *
    *************************/
    if (cinfo == (struct tbl_colinfo* ) NULL)
    {
	cinfo = (struct tbl_colinfo *)
	malloc(sizeof(struct tbl_colinfo ));
	if (cinfo == (struct tbl_colinfo *)NULL) return(TBL_NOMEM);
    }
    file_no   = tbl_finfo[ifile]->file_handle;
    tabletype = tbl_finfo[ifile]->table_type;


    /*******************
    * ISIS ASCII TABLE *
    ********************/
    if (tabletype == ISIS_ASCII)
    {
	strcpy(cinfo->name,       tbl[file_no]->colinfo[icol]->name);
	strcpy(cinfo->unit,       tbl[file_no]->colinfo[icol]->unit);
	strcpy(cinfo->data_type,  tbl[file_no]->colinfo[icol]->dtyp);
	strcpy(cinfo->null_string,tbl[file_no]->colinfo[icol]->null);
	strcpy(cinfo->scale,      tbl[file_no]->colinfo[icol]->scal);
	strcpy(cinfo->offset,     tbl[file_no]->colinfo[icol]->zero);
	strcpy(cinfo->display,  " ");
	cinfo->byte_width  = tbl[file_no]->colinfo[icol]->width;
	cinfo->endcol      = tbl[file_no]->colinfo[icol]->endcol;
    }
  return(TBL_OK);
}

