/****************************************************************************
**                                                                         **
** Routine Name:                tbl_colstr.c                               **
**                                                                         **
**                                                                         **
** Description:                 This routine reads a required data record. **
**				It returns column value in character       **
**			        string.                                    **
**                                                                         **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              erow    ____ row number want to read       **
**                              col     ____ column number                 **
**                              *stat   ____ return status                 **
**                                                                         **
**                                                                         **
** Output Parameters:           *value  ---- string at (erow,col) 	   **
**                                                                         **
**                                                                         **
** Function Return Values:      *stat = TBL_OK     (0), success            **
**                              *stat = TBL_NOMEM (-2), no memory left     **
**                              *stat = TBL_EOF   (-4), end of file        **
**                              *stat = TBL_RDERR (-5), read error         **
**                              *stat = TBL_NOTOPN(-6), file not open      **
**                                                                         **
**                                                                         **
** Usage:			char *tbl_colvals(int ifile, int erow,     **
**				            int col, int* stat)            **
**				                                           **
**				                                           **
** Routines Be Called:	        FITS library calls:      		   **
**				     ffgcve.c 				   **
**				     ffgcvi.c 				   **
**				     ffgcvui.c 				   **
**				     ffgcvj.c 				   **
**				     ffgcvuj.c 				   **
**				     ffgcvd.c 				   **
**				     ffgcvs.c 				   **
**                                                                         **
** 				ISIS library calls:                        **
**				     isis_read.c			   **
**				     isis_colstr.c			   **
**				                			   **
**				                			   **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    first written				   **
**                                                                         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/*************************
* define global variables *
**************************/
double      *tscal[TBL_MAXFILES];       /* scale factor */ 
double      *tzero[TBL_MAXFILES];       /* data offset */

long int    tbcol[TBL_MAXFILES][200];   /* ASCII table ending col. byte num. */
long int    key_bytes[TBL_MAXFILES];    /* number of bytes in the header */
long int    rec_bytes[TBL_MAXFILES];    /* number of bytes in a record */
long int    isis_add[TBL_MAXFILES];     /* offset bytest of an ISIS file */
long int    buffer_size[TBL_MAXFILES];  /* file size */

int         tabletype;          /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */
int         keynum[TBL_MAXFILES];       /* key number */

char        **ttype[TBL_MAXFILES];      /* data type */
char        **tform[TBL_MAXFILES];      /* data format */
char        **tunit[TBL_MAXFILES];      /* data unit */
char        **tdisp[TBL_MAXFILES];      /* display format */
char	    *tbl_fval;		/* character string for FITS column content */
char	    *tbl_ival;	        /* character string for ISIS column content */
char	    fix[TBL_MAXFILES];	/* fix length ISIS ASCII file */


char *tbl_colstr(ifile, erow, col, stat)
int	ifile;			/* file index */
int	erow;			/* row number */
int	col;			/* col number */
int	*stat;			/* status     */
{
    /*************************
    * define local variables *
    **************************/
    int	    		tfields;/* number of column in a file */
    int	    		ii, i;	/* loop index */
    int     		anynull;/* indicate null value */  
    int     		type;	/* column data type */
    int     		file_no;/* file index */

    unsigned short int	fui;	/* dummy variable */
    unsigned long int	fuj;	/* dummy variable */

    short int		fi;	/* dummy variable */
    long int		fj;	/* dummy variable */
    long int	nulval;   /* mumerical value to represent undefined pixel */
    long int    repeat;   /* length of column vector */
    long long int	lli;	/* dummy variable */

    float   		fx;	/* dummy varaible */
    double  		fd;	/* dummy variable */

    char        typechar; /* symbolic code of the table column */
    unsigned char	fb;	/* dummy variable */

    char* isis_colstr(int, int, int*);

    /*
    ##############################
    # read a file and get column #
    # value in character string  #
    ##############################
    */

    /*****************
    * initialization *
    ******************/
    tabletype = tbl_finfo[ifile]->table_type;
    file_no   = tbl_finfo[ifile]->file_handle;
    *stat = 0;

    /*******************
    * ISIS ASCII table *
    ********************/
    if (tabletype == ISIS_ASCII)
    {
	if (fix[file_no] == 'T')
	{

	    isis_add[file_no] = key_bytes[file_no] 
			      + erow * rec_bytes[file_no];
	    if (isis_add[file_no] >= buffer_size[file_no]) 
	    {
		*stat = TBL_EOF;
	        sprintf(tbl_ival, "TBL_EOF");
	        return(tbl_ival);
            }
        }
	else
        {
            *stat    = isis_read(file_no, erow);    /* read a record at row erow */
             if (*stat < 0)
	     {
	        sprintf(tbl_ival, "TBL_EOF");
	        return(tbl_ival);
             }
        }
	tbl_ival = isis_colstr(file_no, col, stat); /* get string at column col. */
	return(tbl_ival);			    /* return string */ 
    }

}
