/****************************************************************************
**                                                                         **
** Routine Name:                tbl_column.c                               **
**                                                                         **
**                                                                         **
** Description:                 This routine returns corresponding column  **
**                              number for a given column name, if such a  **
**                              column exists.				   **
**                                                                         **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              *name   ____ name of the column            **
**                                                                         **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
**                                                                         **
** Function Return Values:      stat > 0,                  column number   **
**                              stat = TBL_OK       (  0)  success         **
**                              stat = TBL_ERROR    ( -1)  error           **
**                              stat = TBL_NOCOL    ( -7)  no such col.    **
**                                                                         **
**                                                                         **
** Usage:			tbl_column(int ifile, char * name)         **
**                                                                         **
**                                                                         **
** Routines Be Called:   	FITS library calls:                        **
**                                   ffgcnn.c                              **
**                                   ffgcno.c                              **
**									   **
**				ISIS library calls:			   **
**                                   isis_column.c                         **
**                                   isis_rewind.c                         **
**									   **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    first written                              **
**                                                                         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/**************************
* Define Global Variables *
***************************/
int     tabletype;              /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */


tbl_column(ifile, name)
int 	ifile;			/* input file number */	 
char 	*name;			/* pointer to char */	 


{
  /*************************
  * Define Local Variables *
  **************************/
  int	icol;			/* column number */
  int	stat;
  int	file_no;		/* file index */

  /************************
  * Check Column name and *
  * return the number     *
  *************************/

    stat = 0;
    file_no  = tbl_finfo[ifile]->file_handle;
    tabletype= tbl_finfo[ifile]->table_type;

    /*------------------
    | ISIS ASCII table |
    -------------------*/
    if (tabletype == ISIS_ASCII)
    { 
	icol = isis_column(file_no, name);
	if (icol < 0) return(TBL_ERROR);
    }
    return(icol);
}

