/***********************
* define include files *
************************/
#include <stdio.h>

/********************
* define error type * 
*********************/
#define     TBL_OK           0
#define     TBL_ERROR  	    -1
#define     TBL_NOMEM  	    -2
#define     TBL_NOFILE 	    -3
#define     TBL_EOF    	    -4
#define     TBL_RDERR  	    -5
#define     TBL_NOTOPN      -6
#define     TBL_NOCOL  	    -7
#define     TBL_TYPERR      -8
#define     TBL_NOTYP  	    -9
#define     TBL_CNVERR     -10
#define     TBL_WSCAL  	   -11
#define     TBL_WARNING      4	


/*******************
* define data type * 
********************/
#define     TBL_DOUBLE       1	
#define     TBL_INTEGER      2	
#define     TBL_CHARACTER    3	


/******************
* define constant * 
*******************/
#define     TBL_MAXSTR    4096
#define     TBL_MAXHSTR    800
#define     TBL_MAXCOLS    500 
#define     TBL_MAXFILES    10
#define     TBL_COMMENT     72
#define     WIDTH            0
#define     ENDCOL           0
#define     HEADER           0
#define     TBL_MAXSUBTIT    6
#define     TBL_MAXSUBLEN   80
#define     TBL_KEYNUM      1000  
#define     TBL_KEYLEN      256 


/********************
* define table type * 
*********************/
#define     ISIS_ASCII       3

/********************
* define macros     * 
*********************/
#define iswhite(c) (*c==' '||*c=='\t')
#define skipwhite(c)   while (*c != '\0' && iswhite(c)) c++

/*******************
* define structure * 
********************/
struct colstruct
{
   int      maxname;
   int      endcol;
   int      width;
   int      precis;
   int      delcol;
   int      update;
   char     *name;
   char     *dtyp;
   char     *unit;
   char     *null;
   char     *scal;
   char     *zero;
};

struct colvalstruct
{
   int    nelem;
   int    *iptr;
   double *dptr; 
   char   **cptr;
};

struct keyword
{
   int      ktype;
   char     key[TBL_KEYLEN];
   char     kval[TBL_KEYLEN];
   char     kcomment[TBL_COMMENT];
};

struct tblstruct
{
   FILE     *tfile;
   int      header;
   int      reclen;
   char     *rec;
   int      maxcols;
   int      ncols;
   int      *delrow;
   struct   colvalstruct   **colvalue;
   struct   colstruct      **colinfo;
   struct   keyword        **keyinfo;
   int      keynum;
};

struct tbl_filinfo
{
   char          file_name[TBL_MAXSTR];
   int           file_handle;
   FILE          *fileptr;
   int           table_type;
   int           link_handle;
   long int      nrows;
   int           ncols;
   int           nkeywords;
};


struct tbl_colinfo
{
   int           byte_width;
   int           precis;     
   int           endcol;
   char          name[TBL_MAXHSTR];
   char          data_type[TBL_MAXHSTR];
   char          unit[TBL_MAXHSTR];
   char          null_string[TBL_MAXHSTR];
   char          scale[TBL_MAXHSTR];
   char          offset[TBL_MAXHSTR];
   char          display[TBL_MAXHSTR];
};


/**************************
* define global variables * 
***************************/
#ifndef tbl_define_globals
/* most uses of tbl_cpp.h use "extern" as a reference */
extern struct tblstruct    **tbl;
extern struct tbl_filinfo  **tbl_finfo;

extern char        	    tblerror[256];
#else
/*tbl_close.c invokes this form to allocate storage */
struct tblstruct    **tbl=0;
struct tbl_filinfo  **tbl_finfo=0;

char        	    tblerror[256]={0};
#endif


/********************
* define subroutine * 
*********************/
char *tbl_colstr();
char *isis_colstr();
char *isis_error();


extern "C"{
int tbl_open(char* fname, struct tbl_filinfo **passinfo);
int tbl_close(int);
int tbl_colval(int ifile, int row, int col, int type, void **ptr); 
char *tbl_colstr(int ifile, int row, int col, int *stat);
int tbl_fileinfo(int , struct tbl_filinfo  **fileinfo); 
int tbl_keyinfo(int ifile, int ith,  char *key, char *keyvalue);
int tbl_keyval(int ifile, char *key, char *keyvalue, int *ith);
int tbl_keystruc(int ifile, int ikey, struct keyword *kinfo);
int tbl_column(int, char*);
int tbl_colinfo(int ifile, int col, struct tbl_colinfo *cinfo);
int tbl_numcheck(int ifile, int *nrows);
int tbl_openw(char* fname, struct tbl_filinfo **passinfo);
int tbl_delcol(int ifile, int col);
int tbl_delrow(int ifile, int row);
int tbl_link(int ifile, int ofile);
int tbl_setkey(int ofile, char *keyname, char *type, void *keyval);
int tbl_setcol(int ofile, char *name, char *type, char *unit, char *null,
           double scale, double offset,int  width, int precis, int *icol);
int tbl_setcolval_c(int ofile, int col, char *type, int nelem, char *ptr[]);
int tbl_setcolval_f(int ofile, int col, char *type, int nelem, void **ptr);
int tbl_writhdr(int ofile);
int tbl_write(int ofile);

	  }
