/****************************************************************************
**                                                                         **
** Routine Name:                tbl_delrow.c                               **
**                                                                         **
**                                                                         **
** Description:                 This routine marks a data row to be        **
**                              deleted in output linked file.
**                                                                         **
**                                                                         **
** Input Parameters:            ofile   ____output file number             **
**                              row     ____ row # to be deleted           **
**                                                                         **
**                                                                         **
** Output Parameters:           None                                       **
**                                                                         **
**                                                                         **
** Function Return Values:      stat > 0,                  column number   **
**                              stat = TBL_OK       (  0)  success         **
**                              stat = TBL_ERROR    ( -1)  error           **
**                              stat = TBL_NOCOL    ( -7)  no such col.    **
**                                                                         **
**                                                                         **
** Usage:			tbl_column(int ifile, char * name)         **
**                                                                         **
**                                                                         **
** Routines Be Called:   	FITS library calls:                        **
**                                   ffgcnn.c                              **
**                                   ffgcno.c                              **
**									   **
**				ISIS library calls:			   **
**                                   isis_column.c                         **
**                                   isis_rewind.c                         **
**									   **
** Revision History:                                                       **
**                                                                         **
**   2005-01    Iffat Khan      first written                              **
**                                                                         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/**************************
* Define Global Variables *
***************************/
int     tabletype;              /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */


tbl_delrow(ofile, irow)
int 	ofile;			/* output file number */	 
int     irow; 	/* indx of input file row to be deleted */


{
  /*************************
  * Define Local Variables *
  **************************/
  int	stat;
  int   ifile;

  /************************
  * Mark Row    for       *
  * deletion.             *
  *************************/

    stat = 0;
    ifile = tbl_finfo[ofile]->link_handle;
    tbl[ifile]->delrow[irow] = 1;
    return(stat);
}

