/****************************************************************************
**                                                                         **
** Routine Name:                tbl_colinfo.c                              **
**                                                                         **
** Description:                 This routine retrieves a keyword           **
**                              information                                **
** Input Parameters:            					   **
** 				ifile   ____ input file number             **
**                              ikey    ____ keyword index                 **
**                                                                         **
** Output Parameters:                                                      **
**                              *kinfo  ____ keyword information           **
**					     structure  		   **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
** Routines Be Called:          ISIS library calls:			   **
**				     none     			   **
**									   **
**									   **
** Revision History:                                                       **
**									   **
**   2004-04    Iffat  Khan     first written	                           **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Includeing Files *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* Define Global Variables *
***************************/
int     tabletype;              /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */


tbl_keystruc(ifile, ikey, kinfo)

int 		   ifile;	/* input file number */	 
int 		   ikey;	/* key index */	 
struct keyword *kinfo;	/* struct for keyword information */	 

{
    int 	stat;		/* column position index */	 
    int 	i,j;		/* dummy variable */	 
    int 	len;		/* len of the variable */	 
    int 	file_no;	/* file index */	 
    int 	ktype;  	/* file index */	 
    char 	key[TBL_KEYLEN];	/* keyword name */ 
    char 	kval[TBL_KEYLEN];     	/* char value of keyword */ 
    char 	kcomnt[TBL_COMMENT]; /* comment in keyword  */	 


    /************************
    * allocate memory space *
    * for tbl_finfo         *
    *************************/
    if (kinfo == (struct keyword* ) NULL)
    {
	kinfo = (struct keyword *)
	malloc(sizeof(struct keyword ));
	if (kinfo == (struct keyword *)NULL) return(TBL_NOMEM);
    }
  file_no   = tbl_finfo[ifile]->file_handle;


  strcpy(kinfo->key ,       tbl[file_no]->keyinfo[ikey]->key); 
  strcpy(kinfo->kval,       tbl[file_no]->keyinfo[ikey]->kval); 
  strcpy(kinfo->kcomment,  tbl[file_no]->keyinfo[ikey]->kcomment);
  kinfo->ktype  = tbl[file_no]->keyinfo[ikey]->ktype; 
  return(TBL_OK);
}

