/****************************************************************************
**                                                                         **
** Routine Name:                tbl_keyval.c                               **
**                                                                         **
** Description:                 This routine retrieves value of a keyword  **
**                              when a keyword is given.                   **
**                                                                         **
**                                                                         **
** Input Parameters/ output parameters:					   **
**                                                                         **
**				ifile   ____ input file number             **
**  			        key	____ content of keywords	   **
**                                                                         **
** Input Parameters/ output parameters:					   **
**                                                                         **
**  			        kval	____ value of keywords	           **
**  			        indx	____ keyword index                 **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                              stat = TBL_NOTOPN(-6), not open            **
**                              stat = TBL_NOTYP (-9), no such type        **
**                                                                         **
** Usage:								   **
** 				tbl_keyval(int    ifile,                   **
**					    char* key,	                   **
**					    char* kval,                    **
**					    int*  indx)                    **
**									   **
**									   **
** Routines Be Called:          FITS library calls:			   **
** 			 	     ffgkyn				   **
**                                                                         **
**				ISIS library calls:			   **
** 			 	     isis_keyinfo			   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   1998-03    Angela Zhang    initial written                            **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**   2004-06    Iffat Khan      removed leading blanks from key as         **
**                              the stored keywords do not have blanks     **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Includeing Files *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/**************************
* define global variables *
***************************/
char	*tbl_fval;		/* string for FITS column content */
char	*tbl_ival;		/* string for ISIS column content */
int     tabletype;		/* FITS_ASCII, FITS_BINARY, ISIS_ASCII */
int     keynum[TBL_MAXFILES];	/* keywords number */


tbl_keyval(ifile, key, kval, indx)

int 	ifile;			/* input file number */	 
int 	*indx;			/* keyword number */	 
char 	*kval;	                /* keyword value  */	 
char 	key[TBL_KEYLEN]; 	/* keyword */	 

{

    int		stat;		/* status of a procedure */
    int		nkey;		/* number of keyword */
    int		file_no;	/* file number */
    int		i;		/* dummy variable */
    int         noblank;        /* index of 1st non blank char */
    char 	keyword[TBL_KEYLEN];	/* keyword */	 


    /*****************
    * get table type *
    ******************/
    tabletype = tbl_finfo[ifile]->table_type;
    file_no   = tbl_finfo[ifile]->file_handle;
    nkey      = tbl_finfo[ifile]->nkeywords;

    /********************
    * get key value for *
    * a given keyword   *
    *********************/
    strcpy(keyword,key);
    noblank = 0;
    for (i=0; i<strlen(keyword); i++)
      if (keyword[i] != ' ')
        { noblank = i; break; }
    if (tabletype == ISIS_ASCII)
    { 
	for (i = 0; i < nkey; ++i)
	{
	     if (!strcmp(tbl[file_no]->keyinfo[i]->key, keyword+noblank)) break;
	}
        *indx = i+1;
        strcpy(kval, tbl[file_no]->keyinfo[i]->kval);
	if (i == nkey)
	{
	    strcpy(kval," ");
            *indx = -1;
	    return(TBL_ERROR);
        }
    }
    else 
	return (TBL_NOTYP);

    return(TBL_OK);
}

