/****************************************************************************
**                                                                         **
** Routine Name:                tbl_numcheck.c                             **
**                                                                         **
**                                                                         **
** Description:                 This routine returns number of rows in     **
**				a table file.                              **
**                                                                         **
**                                                                         **
** Input Parameters:            ifile   ____ input file number             **
**                              stat   ____ return status                  **
**                                                                         **
**                                                                         **
** Output Parameters:           *erow  ---- number of rows in a file 	   **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK     (0), success             **
**                              stat = TBL_NOMEM (-2), no memory left      **
**                              stat = TBL_EOF   (-4), end of file         **
**                              stat = TBL_RDERR (-5), read error          **
**                              stat = TBL_NOTOPN(-6), file not open       **
**                                                                         **
**                                                                         **
** Usage:			stat = tbl_colvals(int ifile, int* erow)   **
**				                                           **
**				                                           **
** Routines Be Called:	       					           ** 
**                                                                         **
** 				ISIS library calls:                        **
**				     isis_read.c			   **
**				     isis_colstr.c			   **
**				                			   **
**				                			   **
** Revision History:                                                       **
**                                                                         **
**   1998-10    Angela Zhang    first written				   **
**                                                                         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

/*************************
* define global variables *
**************************/
int         tabletype;          /* FITS_ASCII, FITS_BINARY, or ISIS_ASCII */
char	    fix[TBL_MAXFILES];	/* fix length ISIS ASCII file */

tbl_numcheck(ifile, erow)
int	ifile;			/* file index */
int	*erow;			/* row number */
{
    /*************************
    * define local variables *
    **************************/
    int     		file_no;/* file index */
    int	                stat;	/* status     */

    /*****************
    * Initialization *
    ******************/
    tabletype = tbl_finfo[ifile]->table_type;

    /*******************
    * ISIS ASCII table *
    ********************/
    if (tabletype == ISIS_ASCII && fix[ifile] != 'T')
    {

        /*******************
        * ISIS ASCII table *
        ********************/
        *erow = 0;
        stat  = 0;
        while (stat == 0)
        {
            stat  = isis_read(ifile, *erow);
            if (stat <0 ) return(stat);
            *erow = *erow + 1;
        }
        tbl_finfo[ifile]->nrows = *erow - 1;
    }
    *erow = tbl_finfo[ifile]->nrows;
    return(TBL_OK);
}
