/****************************************************************************
**                                                                         **
** Routine Name:                tbl_openw.c                                **
**                                                                         **
**                                                                         **
** Description:                 This routine opens an output file which    **
**                              will be an IPAC ASCII.                     **
**                                                                         **
**                                                                         **
** Input Parameters:            *fname   ____ output file name             **
**                                                                         **
**                                                                         **
**                                                                         **
** Function Return Values:      istat = file number(>0), suceses           **
**                              istat = TBL_ERROR  (-1), error             **
**                              istat = TBL_NOMEM  (-2), no memory left    **
**                              istat = TBL_RDERR  (-5), read rec. error   **
**                              istat = TBL_NOTOPN (-6), can't open file   **
**                              istat = TBL_TYPERR (-8), not a proper file **
**                                                                         **
**                                                                         **
** Usage:			tbl_openw(char* fname, 			   **
**					 struct tbl_filinfo **passinfo)	   **
**									   **
**									   **
**				     					   **
**				ISIS library calls:			   **
**				     isis_open.c			   **
**				     isis_read.c			   **
**									   **
**									   **
** Revision History:                                                       **
**                                                                         **
**   2004-02    Iffat Khan      code first written		           **
**   2004-08    Iffat Khan      modified for appending in input file       **
**   2005-10    Iffat Khan      used a different approach for temp file    **
**                              (ehile appending data in input file)       **
**                                                                         **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/*
##########################
# define including files #
##########################
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <tbl.h>
#include <sys/stat.h>
#include <sys/types.h>


/*
###########################
# define global variables #
###########################
*/
char  mode;

tbl_openw(fname, passinfo)    
char		    *fname;	/* input file name */
struct tbl_filinfo  **passinfo; /* file information structure */

{
  int ofile, jfile;
  int ndum = 0;
  int stat;
  int  tmp_out = 0;

    if (tbl_finfo == (struct tbl_filinfo **) NULL)
    {
        tbl_finfo = (struct tbl_filinfo **)
                 malloc(TBL_MAXFILES * sizeof(struct tbl_filinfo *));
        if (tbl_finfo == (struct tbl_filinfo **)NULL) return(TBL_NOMEM);
   
        for (jfile = 0; jfile < TBL_MAXFILES; ++jfile)
        {
             tbl_finfo[jfile] = (struct tbl_filinfo *)
                                 malloc( sizeof(struct tbl_filinfo));
             if (tbl_finfo[jfile] == (struct tbl_filinfo *) NULL)
                                  return(TBL_NOMEM);
             tbl_finfo[jfile]->fileptr = (FILE *) NULL;
        }
    }

    /************************
    * find file number *
    ********************/

    for (jfile = 0; jfile < TBL_MAXFILES; ++jfile)
    {
         if (tbl_finfo[jfile]->fileptr == (FILE *) NULL)
             break;
    }

    mode = 'w';
    ofile  = isis_open(fname,  &ndum);       
    if (ofile < 0) return(TBL_ERROR);

/************************
 * save file information *
*************************/
    strcpy(tbl_finfo[jfile]->file_name, fname);
    tbl_finfo[jfile]->fileptr      = tbl[ofile]->tfile;
    tbl_finfo[jfile]->table_type   = ISIS_ASCII;
    tbl_finfo[jfile]->file_handle  = ofile;
    tbl_finfo[jfile]->link_handle  = -1;
    tbl_finfo[jfile]->nkeywords = 0;
    tbl_finfo[jfile]->ncols = 0;

/******************************
* Allocate Memory for Keyword *
*******************************/

  stat   = isis_keymallc(ofile);
  if (stat < 0) return(stat);


return(ofile);
}
