/****************************************************************************
**                                                                         **
** Routine Name:                tbl_setcol.c                               **
**                                                                         **
** Description:                 This routine defines a new column          **
**                                                                         **
** Input Parameters:            					   **
** 				ofile   ____ output file number            **
**                              data_type    data type integer             **
**                              unit    ____ unit of data                  **
**                              null    ____ null value, if any            **
**                              scale    ____ scale                        **
**                              offset   ____ offset                       **
**                              width    ____ width of the col             **
**                              precs    ____ presion to be used           **
**                                                                         **
** Output Parameters:                                                      **
**                              icol    ____ column number                 **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success             **
**                              stat = TBL_ERROR (-1), error               **
**                                                                         **
**									   **
** Revision History:                                                       **
**									   **
**   2004-02    Iffat Khan      first written	                           **
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/**************************
* define Including Files  *
***************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>

#define MAX(a,b) (((a) > (b)) ? (a) : (b))


/**************************
* Define Global Variables *
***************************/


tbl_setcol(ofile, name, type, unit, null, scale, offset,
           width, precis,  icol)

int    ofile;	/* input file number */	 
char   *name;
char   *type;
char   *unit, *null;
double scale, offset;
int    width, precis;
int   *icol;	/* output column index */	 

{
    int         file_no;
    long int 	ncol;		/* column position index */	 
    int         col_width;      /* column width */
    int         name_size;      /* length of input name string */
    int         unit_size;      /* length of input unit string */
    int         null_size;      /* length of input null string */
    char        ktype;

    if (tbl_finfo == (struct tbl_filinfo **) NULL)
    {
        tbl_finfo = (struct tbl_filinfo **)
                 malloc(TBL_MAXFILES * sizeof(struct tbl_filinfo *));
        if (tbl_finfo == (struct tbl_filinfo **)NULL) return(TBL_NOMEM);

    }
    skipwhite(type);
    ktype = tolower(type[0]);
    if (ktype != 'f' && ktype != 'r' &&
        ktype != 'd' && ktype != 'i' &&
        ktype != 'c' ) {
          printf("Unknown data type\n");
          return(TBL_ERROR);
    }

    file_no   = tbl_finfo[ofile]->file_handle;
    ncol = tbl[file_no]->ncols;
    ncol++;	
    tbl[file_no]->ncols = ncol;
    tbl_finfo[ofile]->ncols = ncol;
    ncol--;
    *icol = ncol;

    if (scale == 0.0) scale = 1.0;

    /*******************
    * SAVE PARAM SPECS *
    ********************/

  name_size = strlen(name);
  unit_size = strlen(unit);
  null_size = strlen(null);

  col_width = name_size;
  if (unit_size > col_width) col_width = unit_size;   
  if (null_size > col_width) col_width = null_size;
   
  
/*check name string */
  if (name_size < TBL_MAXHSTR)
     strcpy(tbl[file_no]->colinfo[ncol]->name, name);
  else
  {
     printf("Warning: input name [%s]\n is greater than the max allowable length. Truncating..\n",name);
     strncpy(tbl[file_no]->colinfo[ncol]->name, name, TBL_MAXHSTR-1);
     tbl[file_no]->colinfo[ncol]->name[TBL_MAXHSTR-1] = '\0';
  }

/*check unit string */
  if (unit_size < TBL_MAXHSTR)
     strcpy(tbl[file_no]->colinfo[ncol]->unit, unit);
  else
  {
     printf("Warning: input unit [%s]\n is greater than the max allowable length. Truncating..\n",unit);
     strncpy(tbl[file_no]->colinfo[ncol]->unit, unit, TBL_MAXHSTR-1);
     tbl[file_no]->colinfo[ncol]->unit[TBL_MAXHSTR-1] = '\0';
  }

/*check null string */
  if (null_size < TBL_MAXHSTR)
     strcpy(tbl[file_no]->colinfo[ncol]->null, null);
  else
  {
     printf("Warning: input null [%s]\n is greater than the max allowable length. Truncating..\n",null);
     strncpy(tbl[file_no]->colinfo[ncol]->null, null, TBL_MAXHSTR-1);
     tbl[file_no]->colinfo[ncol]->null[TBL_MAXHSTR-1] = '\0';
  }

  strcpy(tbl[file_no]->colinfo[ncol]->dtyp, type);
  sprintf(tbl[file_no]->colinfo[ncol]->scal,"%f", scale);
  sprintf(tbl[file_no]->colinfo[ncol]->zero,"%f", offset);

/* reset col width */
  if (col_width > width) width = col_width;
  if (width > TBL_MAXHSTR) width = TBL_MAXHSTR; 
  tbl[file_no]->colinfo[ncol]->endcol = width + 1;
  tbl[file_no]->colinfo[ncol]->width = width;
  tbl[file_no]->colinfo[ncol]->precis = precis ;

  return(TBL_OK);
}

