/**************************************************************************** 
**                                                                         **
** Routine Name:                tbl_setcolval_c.c                          **
**                                                                         **
**                                                                         **
** Description:                 This routine set the values of a new       **
**                              character type column.                     **
**                                                                         **
** Input Parameters:            ofile   ____ input file number             **
**                              col     ____ column number                 ** 
**                              type    ____ data type of he column        **
**                              nelem   ____ no of elements in input array **
**                              **ptr   ____ void  data pointer            **
**                                                                         **
**                                                                         **
** Output Parameters:           None   				 	   **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK    (  0) success             **
**                              stat = TBL_WARNING( 4) can't be scaled     **
**                              stat = TBL_ERROR ( -1) error               **
**                              stat = TBL_NOOPN ( -6) file does not open  **
**                              stat = TBL_NOCOL ( -7) no such column      **
**                              stat = TBL_NOTPY ( -9) no such data type   **
**                              stat = TBL_CNVERR(-10) convertion error    **
**                              stat = TBL_WSCAL (-11) zero scaling factor **
**                                                                         **
**                                                                         **
** Usage:			tbl_setcolval_c(int ofile, int col,        **
**			                   int type, int nelem, void **ptr)**
**                                                                         **
**                                                                         **
**                                                                         **
**				     ISIS Routine Calls                    **
**				     None                     	           **
**                                                                         **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   2004-03    Iffat Khan      code first written	                   ** 
**   2005-10    Iffat Khan      check for data type, case insensitive ** 
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* define global variables *
***************************/
char 	*tbl_ival;		  /* string for ISIS column content */
char    **tform[TBL_MAXFILES];    /* data format */


tbl_setcolval_c(ofile, col, type, nelem, ptr)
int	ofile;			  /* file index */
int	col;			  /* col number */
char    *type;			  /* data type  */
int	nelem;		          /* no of elements in given array */
char    *ptr[];			  /* data pointer */

{
    /*************************
    * define local variables *
    **************************/
    int	    		inttype;   /* return status */
    int	    		stat;	   /* return status */
    int	    		value_size; /* number of column */
    int	    		ii, i;	   /* loop index */
    int     		anynull;   /* null value */	
    int     		file_no;   /* file index */	

    char		**cptr;     /* character data pointer */
    char                p;         /* dummy variable */
    char                str[TBL_MAXSTR];


    stat = 0;
    file_no    = tbl_finfo[ofile]->file_handle;

/* first allocate array */
    if (tbl[ofile]->colvalue[col]->nelem == 0)
      tbl[ofile]->colvalue[col]->nelem = nelem;
    else if (tbl[ofile]->colvalue[col]->nelem != nelem) {
      printf("Inconsistent no of elements for output columns\n");
      return(TBL_ERROR);
    }     

   skipwhite(type);
   p = tolower(type[0]);
   if (p != tolower(tbl[ofile]->colinfo[col]->dtyp[0])) {
     printf("data type is not the column data type\n");
     return(TBL_ERROR);
   }
    value_size  = nelem * sizeof(char *);
    tbl[ofile]->colvalue[col]->cptr
              = (char **)malloc(value_size);
    if (tbl[ofile]->colvalue[col]->cptr == (char **)NULL)
    {
       sprintf(tblerror,"Cannot malloc () enough memory.");
       return (TBL_NOMEM);
    }
    value_size  = TBL_MAXSTR * sizeof(char);
    for(i=0; i<nelem; i++) {
       tbl[ofile]->colvalue[col]->cptr[i]
                    = (char *)malloc(value_size);
       if (tbl[ofile]->colvalue[col]->cptr[i] == (char *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }
    }
        for(i=0; i< nelem; i++)
        {
          strcpy(str, ptr[i]);
          strcpy(tbl[ofile]->colvalue[col]->cptr[i], str);     
        }
   return(TBL_OK);

}     
