/**************************************************************************** 
**                                                                         **
** Routine Name:                tbl_setcolval_f.c                          **
**                                                                         **
**                                                                         **
** Description:                 This routine set the values of a new       **
**                              floating type column.                      **
**                                                                         **
** Input Parameters:            ofile   ____ input file number             **
**                              col     ____ column number                 ** 
**                              type    ____ data type of he column        **
**                              nelem   ____ no of elements in input array **
**                              **ptr   ____ void  data pointer            **
**                                                                         **
**                                                                         **
** Output Parameters:           None   				 	   **
**                                                                         **
**                                                                         **
** Function Return Values:      stat = TBL_OK    (  0) success             **
**                              stat = TBL_WARNING( 4) can't be scaled     **
**                              stat = TBL_ERROR ( -1) error               **
**                              stat = TBL_NOOPN ( -6) file does not open  **
**                              stat = TBL_NOCOL ( -7) no such column      **
**                              stat = TBL_NOTPY ( -9) no such data type   **
**                              stat = TBL_CNVERR(-10) convertion error    **
**                              stat = TBL_WSCAL (-11) zero scaling factor **
**                                                                         **
**                                                                         **
** Usage:			tbl_setcolval_f(int ofile, int col,        **
**			                   int type, int nelem, void **ptr)**
**                                                                         **
**                                                                         **
**                                                                         **
**				ISIS library calls:None 	   **
**                                                                         **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   2004-03    Iffat Khan      code first written	                   ** 
**   2005-10    Iffat Khan      check for data type, case insensitive ** 
**                                                                         **
** Version:                     TBL0.0                                     **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/

/***********************
* define Include Files *
************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>


/**************************
* define global variables *
***************************/
char 	*tbl_ival;		  /* string for ISIS column content */
char    **tform[TBL_MAXFILES];    /* data format */


tbl_setcolval_f(ofile, col, type, nelem, ptr)
int	ofile;			  /* file index */
int	col;			  /* col number */
char    *type;			  /* data type  */
int	nelem;		          /* no of elements in given array */
void    **ptr;			  /* data pointer */

{
    /*************************
    * define local variables *
    **************************/
    double  		*dptr;	   /* double data pointer */
    float               *fptr;

    int	    		stat;	   /* return status */
    int	    		value_size; /* number of column */
    int	    		ii, i;	   /* loop index */
    int     		anynull;   /* null value */	
    int     		file_no;   /* file index */	
    char                p;         /* dummy variable */


    stat = 0;
    file_no    = tbl_finfo[ofile]->file_handle;

/* first allocate array */
    if (tbl[ofile]->colvalue[col]->nelem == 0)
      tbl[ofile]->colvalue[col]->nelem = nelem;
    else if (tbl[ofile]->colvalue[col]->nelem != nelem) {
      printf("Inconsistent no of elements for output columns\n");
      return(TBL_ERROR);
    }     

   skipwhite(type);
   p = tolower(type[0]);
   if (p != tolower(tbl[ofile]->colinfo[col]->dtyp[0])) {
     printf("data type is not the column data type\n");
     return(TBL_ERROR);
   }
   if (p == 'f' || p == 'r') {
       value_size  = nelem * sizeof(double);
       tbl[ofile]->colvalue[col]->dptr
                    = (double *)malloc(value_size);
       if (tbl[ofile]->colvalue[col]->dptr == (double *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

        fptr   = (float *) ptr;
        for(i=0; i< nelem; i++) 
          tbl[ofile]->colvalue[col]->dptr[i] =  fptr[i];
   }
   else if (p == 'd') {
       value_size  = nelem * sizeof(double);
       tbl[ofile]->colvalue[col]->dptr
                    = (double *)malloc(value_size);
       if (tbl[ofile]->colvalue[col]->dptr == (double *)NULL)
       {
           sprintf(tblerror,"Cannot malloc () enough memory.");
           return (TBL_NOMEM);
       }

        dptr   = (double *) ptr;
        for(i=0; i< nelem; i++)
          tbl[ofile]->colvalue[col]->dptr[i] =  dptr[i];
   }
   else
        return(TBL_NOTYP);

   return(TBL_OK);

}     
