/****************************************************************************
** Routine Name:                tbl_write.c                                **
**                                                                         **
** Description:                 This routine writes the header             **
**                              and the data in the output file            **
**                                                                         **
** Input Parameters:            ofile   ____ output file number            **
**                                                                         **
** Output Parameters:                   ____                               **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success		   **
** 			 	stat = TBL_ERROR (-1), error     	   **
**                                                                         **
** Routines Be Called:		tbl_writhdr.c           		   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   2004-02    Iffat Khan      Initial author                             **
**   2004-08    Iffat Khan      modified for appending in input file       **
**   2005-10    Iffat Khan      used a different approach for temp file    **
**                              (appending input file)                     **
**                                                                         **
** Version:                     ISIS0.0                                    **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>



/**************************
* Declear global variables *
***************************/
long int  key_bytes[TBL_MAXFILES];	/* total keyword bytes */
long int  rec_bytes[TBL_MAXFILES];	/* total bytes in a record */
int       tmp_output[TBL_MAXFILES];   /* option to indicate temp file output */
char      fix[TBL_MAXFILES];		/* fixed ISIS_ASCII file */


tbl_write(ofile)
int 	ofile;	 			/* file number */


{
   /**************************
   * Declear local variables *
   ***************************/
   int  file_no;                        /* file no */
   int  link_no;                        /* output file linked? */
   int  ifile;                          /* linked input file ? */
   int	i, j, ch;			/* loop index */
   int	irow;				/* row index  */
   int	nc;      			/* column length */
   int	rec_len;		        /* string length */
   int	width;		                /* col width */
   int	precis;	                        /* data precision */
   int  nelems;                         /* number of data entries */
   int  inelems;                         /* number of data entries for input tbl */
   int	ncols;				/* column numbers in a line */
   int	incols;				/* column numbers in a line for input tbl */
   int	stat;				/* return status */
   int  nrec;
   char	*rec;				/* pointer to rec */
   char	*cvalue;	/* pointer to rec */
   char *dtype,p;
   char type[8];
   char format[12];
   char str[TBL_MAXSTR];


   FILE *tf;				/* file point */
   FILE *tfi;				/* file point */


    tbl_writhdr(ofile);
    incols = ncols = 0;

    /*****************
    * get table info *
    ******************/
    file_no   = tbl_finfo[ofile]->file_handle;
    link_no   = tbl_finfo[ofile]->link_handle;
    tf        = tbl_finfo[ofile]->fileptr;      
    ncols     = tbl_finfo[ofile]->ncols;
    nelems = tbl[ofile]->colvalue[0]->nelem;
    ifile = link_no;
    if (ifile >= 0) {
      incols = tbl_finfo[ifile]->ncols;
      tfi  = tbl_finfo[ifile]->fileptr;      
      stat = isis_rewind(ifile);
      stat = tbl_numcheck(ifile, &inelems);
      if (nelems != inelems) {
        printf("Number of data rows is different in input (%d) and output table (%d)\n"
                , inelems, nelems);
        return(TBL_ERROR);
      }
    }

   /*****************
   * write data     *
   ******************/

/* Data */
  rec = tbl[ofile]->rec;
  for(j=0; j<nelems; j++)
  {
    if (ifile >= 0 && tbl[ifile]->delrow[j]) continue;
    rec[0]=' ';
    nc = 1;

/* input file cols */
   for (i=0; i<incols; i++)
   {
     if (tbl[ifile]->colinfo[i]->delcol) continue;
     stat = 0;
     width = tbl[ifile]->colinfo[i]->width;
     cvalue = tbl_colstr(ifile, j, i, &stat);
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,cvalue);                            
     rec[nc+width] = ' ';
     nc = nc+width+1;
   } 
     
/* write/append new cols */
   for (i=0; i<ncols; i++)
   {
     dtype = tbl[file_no]->colinfo[i]->dtyp;
     skipwhite(dtype);
     p = tolower(dtype[0]);
     width = tbl[file_no]->colinfo[i]->width;
     precis = tbl[file_no]->colinfo[i]->precis;

     if (p == 'r' || p == 'f') {
       sprintf(format,"%%-%d.%df",width, precis);
       sprintf(rec+nc,format,tbl[file_no]->colvalue[i]->dptr[j]);
     }
     else if (p == 'd') {
       sprintf(format,"%%-%d.%de",width, precis);
       sprintf(rec+nc,format,tbl[file_no]->colvalue[i]->dptr[j]);
     }
     else if (p == 'i') {
       sprintf(format,"%%-%dd",width);
       sprintf(rec+nc,format,tbl[file_no]->colvalue[i]->iptr[j]);
     } 
     else if (p == 'c') {
       sprintf(format,"%%-%ds",width);
       tbl[file_no]->colvalue[i]->cptr[j][width] = '\0';
       sprintf(rec+nc,format,tbl[file_no]->colvalue[i]->cptr[j]);
     } 
     rec[nc+width] = ' ';
     nc = nc+width+1;
   }
   fprintf(tf,"%s\n",rec);
 }

/* update the input file */
  if (tmp_output[ofile] && link_no >= 0)
  {
     fclose(tfi);
     tfi = 0;
     tfi = fopen(tbl_finfo[link_no]->file_name,"w");
     if (tfi == NULL)
     {
        printf("Error in opening input file for writing\n");
        return(TBL_ERROR);
     }
     rewind(tf);
     rewind(tfi);
     nrec = 0;
     while (fgets(str, TBL_MAXSTR,tf) != NULL) 
     {
       fputs(str, tfi);
       nrec++;
     }
     tbl_finfo[ofile]->fileptr = tf;      
     tbl_finfo[link_no]->fileptr = tfi;      
  }




   return(0);
}
