/****************************************************************************
** Routine Name:                tbl_writhdr.c                              **
**                                                                         **
** Description:                 This routine writes the header in the      **
**                              output file including keywords and         **
**			 	the lines defining the columns             **
**				specification.                  	   **
**                                                                         **
** Input Parameters:            ofile   ____ output file number            **
**                                                                         **
** Output Parameters:                   ____                               **
**                                                                         **
** Function Return Values:      stat = TBL_OK    ( 0), success		   **
** 			 	stat = TBL_ERROR (-1), error     	   **
**                                                                         **
** Routines Be Called:		None       				   **
**                                                                         **
** Revision History:                                                       **
**                                                                         **
**   2004-02    Iffat Khan      Initial author                             **
**                                                                         **
** Version:                     ISIS0.0                                    **
**                                                                         **
**                                                                         **
** Infrared Science Information System                                     **
** California Institute of Technology                                      **
**                                                                         **
** Copyright (c) 1998, California Institute of Technology.                 **
** U. S. Government Sponsorship under NASA Contract                        **
** NAS7-???? is acknowledged.                                              **
**                                                                         **
*****************************************************************************/


/***********************
* Define Include Files *
************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <tbl.h>



/**************************
* Declear global variables *
***************************/
long int  key_bytes[TBL_MAXFILES];	/* total keyword bytes */
long int  rec_bytes[TBL_MAXFILES];	/* total bytes in a record */
char      fix[TBL_MAXFILES];		/* fixed ISIS_ASCII file */


tbl_writhdr(ofile)
int 	ofile;	 			/* file number */


{
   /**************************
   * Declear local variables *
   ***************************/
   int  file_no;                        /* file no */
   int  link_no;                        /* output file linked? */
   int  file_in;                        /* input file no */
   int  ifile;
   int	i, ch;				/* loop index */
   int	irow;				/* row index  */
   int	nc;      			/* column length */
   int	rec_len;		        /* string length */
   int	width;		                /* record length */
   int  inkeys;                          /* number of keywords defined -input table*/
   int  nkeys;                          /* number of keywords defined */
   int	incols;				/* column numbers in a line -input table*/
   int	ncols;				/* column numbers in a line */
   int	stat;				/* return status */
   int  nent;
   char	*rec;				/* pointer to rec */
   char *p;
   char type[8];
   char format[12];
   char kstr[TBL_KEYLEN];

   float scale;
   float offset;

   FILE *tf;				/* file point */
   FILE *tfin;				/* file point */



    /*****************
    * get table info *
    ******************/
    file_no   = tbl_finfo[ofile]->file_handle;
    link_no   = tbl_finfo[ofile]->link_handle;
    tf        = tbl_finfo[ofile]->fileptr;      
    nkeys     = tbl_finfo[ofile]->nkeywords;
    ncols     = tbl_finfo[ofile]->ncols;
    ifile = link_no;
    if (ifile >= 0) {
      file_in   = tbl_finfo[ifile]->file_handle;
      tfin = tbl[file_in]->tfile;
      if (tfin == (FILE *)NULL)
      {
          sprintf(tblerror, "File not open.");
          return(TBL_ERROR);
      }
      rewind(tfin);
      inkeys     = tbl_finfo[ifile]->nkeywords;
      incols     = tbl_finfo[ifile]->ncols;
      rec        = tbl[ifile]->rec;
      while(1)
      {
        rec_len = isis_gets(ifile);
        if (rec[0] != '\\') break;
        if (rec_len > 0)
          fprintf(tf,"%s\n", rec);
      }
    }

   /*****************
   * write keywords *
   ******************/

   for (i=0; i<nkeys; i++)
   {
     switch(tbl[file_no]->keyinfo[i]->ktype)
     {
       case TBL_DOUBLE:     
         sprintf(kstr,"\\float %s = %s", tbl[ofile]->keyinfo[i]->key,
                 tbl[ofile]->keyinfo[i]->kval);
         break;
       case TBL_INTEGER:     
         sprintf(kstr,"\\int %s = %s", tbl[ofile]->keyinfo[i]->key,
                 tbl[ofile]->keyinfo[i]->kval);
         break;
       case TBL_CHARACTER:   
         if (strcasecmp(tbl[ofile]->keyinfo[i]->key, "comment") == 0 ||
             strcasecmp(tbl[ofile]->keyinfo[i]->key, "history") == 0)

           sprintf(kstr,"\\char %s %s", tbl[ofile]->keyinfo[i]->key,
                 tbl[ofile]->keyinfo[i]->kval);
         else
           sprintf(kstr,"\\char %s = %s", tbl[ofile]->keyinfo[i]->key,
                 tbl[ofile]->keyinfo[i]->kval);
         break;
      }
      fprintf(tf,"%s\n",kstr);
  }
  rec = tbl[ofile]->rec;
  rec[0] = '|';
  nc = 1;
/* name or label */
   if (ifile >= 0) 
   {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->name;
       skipwhite(p);
       width = tbl[ifile]->colinfo[i]->width;
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,p);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
   }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->name;
     skipwhite(p);
     width = tbl[file_no]->colinfo[i]->width;
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,p);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   fprintf(tf,"%s\n",rec);

/* Type */
  rec[0] = '|';
  nc = 1;
  if (ifile >= 0)
  {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->dtyp;
       skipwhite(p);
       width = tbl[ifile]->colinfo[i]->width;
       if (p[0] == 'r')
           strcpy(type,"real");
       else if (p[0] == 'i')
           strcpy(type,"int");
       else if (p[0] == 'f')
           strcpy(type,"float");
       else if (p[0] == 'd')
           strcpy(type,"double");
       else if (p[0] == 'c')
           strcpy(type,"char");
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,type);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
  }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->dtyp;
     skipwhite(p);
     width = tbl[file_no]->colinfo[i]->width;
     if (p[0] == 'r')
         strcpy(type,"real");
     else if (p[0] == 'i')
         strcpy(type,"int");
     else if (p[0] == 'f')
         strcpy(type,"float");
     else if (p[0] == 'd')
         strcpy(type,"double");
     else if (p[0] == 'c')
         strcpy(type,"char");
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,type);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   fprintf(tf,"%s\n",rec);

/* Unit */
  nent = 0;
  rec[0] = '|';
  nc = 1;
  if (ifile >= 0)
  {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->unit;
       skipwhite(p);
       if (strlen(p) > 0)
         nent = 1;
       width = tbl[ifile]->colinfo[i]->width;
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,p);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
  }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->unit;
     skipwhite(p);
     if (strlen(p) > 0)
       nent = 1;
     width = tbl[file_no]->colinfo[i]->width;
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,p);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   if (nent > 0)
     fprintf(tf,"%s\n",rec);

/* Null */
  nent = 0;
  rec[0] = '|';
  nc = 1;
  if (ifile >= 0)
  {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->null;
       skipwhite(p);
       if (strlen(p) > 0)
         nent = 1;
       width = tbl[ifile]->colinfo[i]->width;
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,p);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
   }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->null;
     skipwhite(p);
     if (strlen(p) > 0)
       nent = 1;
     width = tbl[file_no]->colinfo[i]->width;
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,p);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   if (nent > 0)
     fprintf(tf,"%s\n",rec);

/* Scale */
  nent = 0;
  rec[0] = '|';
  nc = 1;
  if (ifile >= 0)
  {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->scal;
       skipwhite(p);
       scale = atof(p);
       if (scale == 1.) p[0] = '\0';
       if (strlen(p) > 0)
         nent = 1;
       width = tbl[ifile]->colinfo[i]->width;
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,p);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
   }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->scal;
     skipwhite(p);
     scale = atof(p);
     if (scale == 1.) p[0] = '\0';
     if (strlen(p) > 0)
       nent = 1;
     width = tbl[file_no]->colinfo[i]->width;
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,p);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   if (nent > 0)
     fprintf(tf,"%s\n",rec);

/* Zeros */
  nent = 0;
  rec[0] = '|';
  nc = 1;
  if (ifile >= 0)
  {
     for (i=0; i<incols; i++)
     {
       if (tbl[ifile]->colinfo[i]->delcol) continue;
       p = tbl[ifile]->colinfo[i]->zero;
       skipwhite(p);
       offset = atof(p);
       if (offset == 0.) p[0] = '\0';
       if (strlen(p) > 0)
         nent = 1;
       width = tbl[ifile]->colinfo[i]->width;
       sprintf(format,"%%-%ds",width);
       sprintf(rec+nc,format,p);
       rec[nc+width] = '|';
       nc = nc+width+1;
     }
   }
   for (i=0; i<ncols; i++)
   {
     p = tbl[file_no]->colinfo[i]->zero;
     skipwhite(p);
     offset = atof(p);
     if (offset == 0.) p[0] = '\0';
     if (strlen(p) > 0)
       nent = 1;
     width = tbl[file_no]->colinfo[i]->width;
     sprintf(format,"%%-%ds",width);
     sprintf(rec+nc,format,p);
     rec[nc+width] = '|';
     nc = nc+width+1;
   }
   if (nent > 0)
     fprintf(tf,"%s\n",rec);

   return(0);
}
