/* tokenize a character string for a namelist item
   sbase is the string to parse
   delims is a string of single-char delimiters (e.g., "=,")
   tmaxlen is the max length of each token result area
   nt_max is the max number of tokens to be returned
   tokens[] is an array of pointers to string storage areas, each one of size tmaxlen
   
   function result - number of tokens in sbase

   pointer snext is updated to the end of the characters scanned for tokens

   warning: quoted text strings have leading, trailing and any double quotes
*/
#include <string.h>
#include <stdio.h>

#include "tokenize.h"

int tokenize(char* sbase, char* delims, int tmaxlen,int nt_max, char* tokens[],char** snext)
{char *s_end,*s,*q,*doublequote,*singlequote;
 int sl,nt,x,y,z,m;
 static char* whitespace =" \n\t";
 static char* quotequote="''";
 static char* quote="'";
 for(x=0;x<nt_max;x++) 
   *(tokens[x])=0;
 nt=0;
 s=sbase;
 while((nt<nt_max) && strlen(s)>0)
   { /* look for end of first whitespace */
     x=strspn(s,whitespace);
     s=s+x;
     /* handle either a token or a quoted character string */
     if (*s==quotequote[0])
       {/* quoted character string , edit into token space eliminating '' */
	 q=tokens[nt];
         while (strlen(s)>0) {
           doublequote=strstr(s,quotequote);
	   if(doublequote==0)
	     {/* no double quote found, terminate and end of string or first quote */
	       m=strlen(s);
	       s_end=s+m;
	       singlequote=strstr(s+1,quote);
	       if(singlequote!=0)
		 {m=singlequote-s+1; /* includes quote(') character */
		 s_end=singlequote+1; 
		 }
             }
           else
             m=doublequote-s+2; /* includes double quote('') characters */
	   y=strlen(q);
           x=y+m;
           if(x>=(tmaxlen-1))
	     m=(tmaxlen-1)-y;
           if(m>0)
             {strncpy(q+y,s,m);
	     z=y+m;
	     if(z<tmaxlen)
	       q[z]=0;
	     }
           if(doublequote!=0)
             {s_end=doublequote+2;
             } 
           else 
             break;            
	   s=s_end;
	 }
       }
     else 
       { /* look for end of nonwhitespace */
	 sl=strcspn(s,whitespace);
	 /* move nonspace items into token storage */
	 if(sl>tmaxlen) 
	   sl=tmaxlen-1;
	 strncpy(tokens[nt],s,sl);
	 tokens[nt][sl]=0;
	 /* look for first delimiter character and trim */
	 x=strcspn(tokens[nt]+1,delims);
	 if((x>=0) && (x<sl))
	   {sl=x+1;
	   tokens[nt][sl]=0;
           s_end=s+sl+1;
	   }
         else {
	 s_end=s+strlen(tokens[nt]);
	 }	
       }
     s=s_end;	
     /* only advance token count if a non-null token encountered */
     if(sl>0)		  
         nt++;
   }
 if(snext!=0)
   (*snext)=s;
 return nt;
}
