#ifndef _tridiagonal
#define _tridiagonal
/* solves a triadiagonal matrix equation 

	A r = u
	
   where:        (following Numberical Recipes section 2.4)
    subdiagonal of A is the a vector, A[1][0]=a[1]
    diagonal of A is the b vector, A[0][0]=b[0]
    superdiagonal of A is the c vector, A[0][1]=c[0]

   Returns:
     -1 if A[0][0] is singular
     -2 if another singularity detected
     0 for solution completed, u has solution
     
   Notes:
   	All arrays are zero origin, gam is a scratch array
   	0th element of the c array is not used
   	scratch array is passed in, is of length n
   	singularity criteria is passed in 
*/
/* $Id: tridiagonal.h,v 1.3 2001/11/12 21:03:32 davidh Exp $ */

#ifdef	__cplusplus
extern "C" {
#endif

int tridiagsolve(float a[], float b[], float c[], float r[], float u[], float gam[],
	float eps, unsigned long n);
/* extract the ith row from a tridiagonal matrix representation */
void trigetrow(float a[], float b[], float c[], float r[],int i, unsigned long n);
/* insert the ith row into a tridiagonal matrix representation */
void triputrow(float a[], float b[], float c[], float r[],int i, unsigned long n);
/* multiply the triangular matrix A times vector x, creating vector rslt */
void trimulvec(float a[], float b[], float c[], float x[], float rslt[], unsigned long n);

int dtridiagsolve(double a[], double b[], double c[], double r[], double u[], double gam[],
	double eps, unsigned long n);
/* extract the ith row from a tridiagonal matrix representation */
void dtrigetrow(double a[], double b[], double c[], double r[],int i, unsigned long n);
/* insert the ith row into a tridiagonal matrix representation */
void dtriputrow(double a[], double b[], double c[], double r[],int i, unsigned long n);
/* multiply the triangular matrix A times vector x, creating vector rslt */
void dtrimulvec(double a[], double b[], double c[], double x[], double rslt[], unsigned long n);

#ifdef	__cplusplus
}
#endif

#endif
