#include <math.h>
/* A portable random number generator takin from June '88 CACM 

   This version works with 32 bit integers.
   
   Usage: 
    1. set the seed of the random number generator with
        uniformseed(longint s1,s2) where 0 < s1,s2 < 214783399
    2. Retrieve the next uniform number between 0 and 1 by
        uniform()
        
    Notes: initialization is optional. Acceptable values are compiled in.
           uniformseed checks the ranges of its arguments and forces acceptable values,
            but it does not complain if unacceptable values are passed in.
*/
#include "ugran.h"    
static long int portrand_s1=123456,portrand_s2=234567;
double uniform_pure()
{long int k1,k2,uniformz;
 k1=portrand_s1/53668L;
 portrand_s1=40014L*(portrand_s1-k1*53668L)-k1*12211;
 if(portrand_s1<0)portrand_s1=portrand_s1+214783563L;
 
 k2=portrand_s2/52774L;
 portrand_s2=40692L*(portrand_s2-k2*52774)-k2*3791L;
 if(portrand_s2<0)portrand_s2=portrand_s2+214783399L;
 
 uniformz=portrand_s1-portrand_s2;
 if(uniformz<1) uniformz=uniformz+2147483562L;
 return(uniformz*4.656613E-10);
}
void uniformseed(long int seed1,long int seed2)
{portrand_s1=seed1;
 portrand_s2=seed2;
 if((portrand_s1<=0)|(portrand_s1>214783399L))portrand_s1=345678L;
 if((portrand_s2<=0)|(portrand_s2>214783399L))portrand_s2=456789L;
}
/* return a normal random number (mean=0, variance=1) from Numerical Recipes */
float gasdev_pure()
{
	static int iset=0;
	static float gset;
	float fac,rsq,v1,v2;

	if  (iset == 0) {
		do {
			v1=2.0*uniform_pure()-1.0;
			v2=2.0*uniform_pure()-1.0;
			rsq=v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	} else {
		iset=0;
		return gset;
	}
}
