/* "C" wrappers for the externally usable methods in C++ class upropclass   
 */

/* $Id: uprop.cpp,v 1.11 2005/06/30 23:03:26 davidh Exp $ */
#include "upropclass.h"
#include "uprop.h"



static upropclass uprop_global;

extern "C" {
  uprop_state uprop_add(float * u, float * usigma, int * umask, float x, float xsigma, int xmask, float y, float ysigma, int ymask)
  {return uprop_global.add(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_sub(float * u, float * usigma, int * umask, float x, float xsigma, int xmask, float y, float ysigma, int ymask)
  {return uprop_global.sub(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_mul(float * u, float * usigma, int * umask, float x, float xsigma, int xmask, float y, float ysigma, int ymask)
  {return uprop_global.mul(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_div(float * u, float * usigma, int * umask, float x, float xsigma, int xmask, float y, float ysigma, int ymask)
  {uprop_state r;
  r=uprop_normal;
  try
    {
      r=uprop_global.div(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);
    }
  catch(char* msg) {}
  return r;
  }
  int uprop_interp(float * u, float * usigma, int * umask, float x, int order, int ny, float *xtab,float *ytab, float *ysigma)
  {int result;
  result=uprop_global.interp(*u, *usigma, *umask, x, order,  ny, xtab, ytab, ysigma);
  return result;
  }
  uprop_state uprop_dadd(double * u, double * usigma, int * umask, double x, double xsigma, int xmask, double y, double ysigma, int ymask)
  {return uprop_global.add(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_dsub(double * u, double * usigma, int * umask, double x, double xsigma, int xmask, double y, double ysigma, int ymask)
  {return uprop_global.sub(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_dmul(double * u, double * usigma, int * umask, double x, double xsigma, int xmask, double y, double ysigma, int ymask)
  {return uprop_global.mul(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);}
  uprop_state uprop_ddiv(double * u, double * usigma, int * umask, double x, double xsigma, int xmask, double y, double ysigma, int ymask)
  {uprop_state r;
  r=uprop_normal;
  try
    {
      r=uprop_global.div(*u,*usigma,*umask,x,xsigma,xmask,y,ysigma,ymask);
    }
  catch(char* msg) {}
  return r;
  }
  int uprop_dinterp(double * u, double * usigma, int * umask, double x, int order, int ny, double *xtab,double *ytab, double *ysigma)
  {int result;
  result=uprop_global.interp(*u, *usigma, *umask, x, order,  ny, xtab, ytab, ysigma);
  return result;
  }

  /* set mask bits for missing, illdef in global data */
  void uprop_mask_setup(int missing_mask_input, int illdef_mask_input)
  {uprop_global.mask_setup(missing_mask_input,  illdef_mask_input,0);
  }
  /* return pointer to global C++ class for default methods */
  upropclass_ptr uprop_globptr()
  {return &uprop_global;}
      
  /* return pointer to global pubdata structure for any C code that needs it */
  uprop_data* uprop_globpubptr()
  {return &uprop_global.pubdata;}

}

/* designed to allow computation modules to function with incomplete file sets-
           missing sigma files are treated as zero sigma's
           missing mask files are treated as zero masks

   returns nonzero if the value file cannot be fetched:
                    file does not exist, the pointer is null, or the value is NaN

   Extremely deliberate:
           do not alter the value "val" if the file or array does not exist.
           (This is used to establish a default value in slopecal module)
*/

int dfetch(double *val, double *vsigma, unsigned short *vmask, int ndx, int missingmask,
	   short exist_val, double *array_val, 
	   short exist_vsigma, double *array_vsigma,
	   short exist_vmask,unsigned short* array_vmask)
{int result=1;
   *vsigma=0;
   *vmask=0;
 if(exist_val && (array_val!=0)) {
   *val=array_val[ndx];
   /* if(! isnan(*val)) AR6578 */
     result=0;
 }
 if(result==0) {
   if(exist_vsigma &&(array_vsigma!=0)) {
     *vsigma=array_vsigma[ndx];
   }
   if(exist_vmask &&(array_vmask!=0)) {
     *vmask=array_vmask[ndx];
   }
   /*  if((*vmask)&missingmask)
     result=1;  AR6578 */
 }
 return result;
}
int ffetch(float *val, float *vsigma, unsigned short *vmask, int ndx, int missingmask,
	   short exist_val, float *array_val, 
	   short exist_vsigma, float *array_vsigma,
	   short exist_vmask,unsigned short* array_vmask)
{int result=1;
   *vsigma=0;
   *vmask=0;
 if(exist_val && (array_val!=0)) {
   *val=array_val[ndx];
   /*   if(! isnan(*val)) AR6578 */
     result=0;
 }
 if(result==0) {
   if(exist_vsigma &&(array_vsigma!=0)) {
     *vsigma=array_vsigma[ndx];
   }
   if(exist_vmask &&(array_vmask!=0)) {
     *vmask=array_vmask[ndx];
   }
   /* if((*vmask)&missingmask)
     result=1;  AR6578 */
 }
 return result;
}
