/* $Id: upropclass.cpp,v 1.18 2006/07/19 22:26:06 davidh Exp $ */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "upropclass.h"
#include "tridiagonal.h"

#include "nanvalue.h"


/* dmask bits - define some to initialize the data structures if not already defined */
#ifndef dmask_missing_default
#ifndef MISSING
#define dmask_missing_default 0x4000
#else
#define dmask_missing_default MISSING
#endif
#endif

#ifndef dmask_illdef_default
#define dmask_illdef_default 0x100
#endif

#ifndef dmask_warning_default
#define dmask_warning_default dmask_missing_default
#endif

/* procedure body for the public "interp" function" */
upropclass::upropclass()
{
  pubdata.CauchyThreshold=9.0;
  pubdata.ApproxThreshold=0.9;
  pubdata.div_exceptions=0;
  mask_setup(dmask_missing_default, dmask_illdef_default);
}
upropclass::upropclass(float CauchyRatio, float ApproxRatio )
{
  pubdata.CauchyThreshold=CauchyRatio;
  pubdata.ApproxThreshold=ApproxRatio;
  pubdata.div_exceptions=0;
  mask_setup(dmask_missing_default, dmask_illdef_default);
}

int upropclass:: interp(float & u, float & usigma, int & umask, float x, int order, int ny, float *xtab,float *ytab, float *ysigma)
{int result;
 if(ny<=0)
   {u=const_nan_float.f;
   usigma=const_nan_float.f;
   return -1;
   }
 else if(ny<=1)
   { u=ytab[0];
   usigma= const_nan_float.f;
   return -2;
   } 
 else if(ny==2)
   { result=linear_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
 else if((order<=2) || (x <xtab[0]) ||(x>xtab[ny-1]))
   { result=linear_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
 else 
   { result=spline_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
}
 
/* linear interpolation
    linear extrapolation when x is outside the tabulated range
 */

int upropclass:: linear_interp(float & u, float & usigma, int & umask, float x, int ny, float *xtab,float *ytab, float *ysigma)
{int result,pos,hi,lo,uval;
 float ratio, tsigmasq;
 
 uval=0;
 result=0;
 
 /* look for abnormal boundary conditions */
 if(ny<2) 
   {result=-3;
   u=const_nan_float.f;
   usigma=const_nan_float.f;
   uval=pubdata.dmask_missing;
   }
 else if(x<xtab[0])
   { /* extrapolation */
     pos=0;
     ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
     u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
     tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
     usigma=sqrt(tsigmasq);
     uval=pubdata.dmask_warning;
     result=1;
   }
 else if(x>xtab[ny-1])
   { /* extrapolation */
     pos=ny-2;
     ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
     u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
     tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
     usigma=sqrt(tsigmasq);
     uval=pubdata.dmask_warning;
     result=1;
   }
 else
   {
     /* binary search for table location  */
     lo=-1;
     hi=ny;
     while ((hi-lo)>1)
       {pos=(lo+hi)>>1;
       if(xtab[pos]<=x)
	 lo=pos;
       else
	 hi=pos;
       }
     pos=lo;
     if(pos==(ny-1))
       pos=ny-2;
     /* linear interpolation */
     if((xtab[pos]<=x) && (x <= xtab[pos+1])  )
       {if(xtab[pos]<xtab[pos+1])
	 {ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
	 u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
	 tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
	 usigma=sqrt(tsigmasq);
	 }
       else
	 {u=ytab[pos];
	 usigma=ysigma[pos];
	 result=0;
	 }
       }  
     else
       {u=const_nan_float.f;
       usigma=const_nan_float.f;
       uval=uprop_missing;
       result=-4;
       }
   }
 if ((uval!=uprop_normal))
   umask|= pubdata.dmask_illdef;
 else
   umask&=~pubdata.dmask_illdef;
 return result;
}
int upropclass::spline_interp(float & u, float & usigma, int & umask, float x, int ny, float *xtab,float *ytab, float *ysigma)
{int result,pos,hi,lo,uval,memerr, Rsolve;

 float A,B,C,D,xdif_ip1,xdif_i,xdif_pos;
   
 float  *R_a, *R_b,*R_c, *S_a, *S_b, *S_c, *gam, *ycoef, *rprod, *ypp_pos1_coef, *ypp_pos_coef ;
 
 /* initialize all auxillary array pointers to NIL for safe cleanup in case of memory exhaustion */
 R_a=R_b=R_c=S_a=S_b=S_c=gam=ycoef=rprod=ypp_pos1_coef=ypp_pos_coef=0;
 memerr=0;

 if(ny>0)
   {
     memerr |=( R_a=new float[ny])==0;
     memerr |=( R_b=new float[ny])==0;
     memerr |=( R_c=new float[ny])==0;
     memerr |=( S_a=new float[ny])==0;
     memerr |=( S_b=new float[ny])==0;
     memerr |=( S_c=new float[ny])==0;
     memerr |=( gam=new float[ny])==0;
     memerr |=( ycoef=new float[ny])==0;
     memerr |=( rprod=new float[ny])==0;
     memerr |=( ypp_pos1_coef=new float[ny])==0;
     memerr |=( ypp_pos_coef=new float[ny])==0;
   }
    
 uval=0;
 result=0;

 /* check for memory allocation error */
 if(memerr!=0)
   {result=-5;
   u=const_nan_float.f;
   usigma=const_nan_float.f;
   uval=uprop_missing;
   }
 
 /* look for abnormal boundary conditions */
 else if( (ny<3) ||  (x<xtab[0]) || (x>xtab[ny-1]))
   {result=-3;
   u=const_nan_float.f;
   usigma=const_nan_float.f;
   uval=uprop_missing;
   }
 else
   {
     /* binary search for table location  */
     lo=-1;
     hi=ny;
     while ((hi-lo)>1)
       {pos=(lo+hi+1)>>1;
       if(xtab[pos]<=x)
	 lo=pos;
       else
	 hi=pos;
       }
     pos=lo;
     if(pos==(ny-1))
       pos=ny-2;
     /* after search and boundary check, 
	0<= pos < ny-1  and xtab[pos] <= x <= x[pos+1] */



     /* spline interpolation 
	1. Calculate tridiagonal R and S matrices
        R y" = S y
	2. Solve for coefs of ypp_j and ypp_jp1
	3. Calculate A,B,C,D
	4. Calculate interpolated yhat
	5. Calculate uncertainty on yhat using coefs
     */

     /* 1. create the triangular-stored R and S matrices */
     ycoef[0]=1.0;
     ycoef[1]=0;
     triputrow(R_a, R_b, R_c, ycoef , 0, ny);
     ycoef[0]=0;
     ycoef[1]=0;
     triputrow(S_a, S_b, S_c, ycoef , 0, ny);

     for(int i=1;i<(ny-1);i++)
       { /* set rprod to be R row i */
	 xdif_i=xtab[i]-xtab[i-1];
	 xdif_ip1=xtab[i+1]-xtab[i];

	 ycoef[i-1]=xdif_i/6.0;
	 ycoef[i+1]=xdif_ip1/6.0;
	 ycoef[i]  = (xtab[i+1]-xtab[i-1])/3.0;
	 triputrow(R_a, R_b, R_c, ycoef , i, ny);

	 ycoef[i-1] = 1.0/xdif_i;
	 ycoef[i]= - 1.0/xdif_ip1 -1.0/xdif_i;
	 ycoef[i+1]=  1.0/xdif_ip1;
	 triputrow(S_a, S_b, S_c, ycoef , i, ny);
       }
     ycoef[ny-1]=1.0;
     ycoef[ny-2]=0;
     triputrow(R_a, R_b, R_c, ycoef , ny-1, ny);
     ycoef[ny-1]=0;
     ycoef[ny-2]=0;
     triputrow(S_a, S_b, S_c, ycoef , ny-1, ny);

     /* 2. Solve for the second derivatives , place in ycoef */
     trimulvec(S_a, S_b, S_c, ytab ,rprod, ny);
     Rsolve=tridiagsolve(R_a, R_b, R_c, rprod, ycoef, gam, 1E-6, ny);
  
     /* check for singular or malformed triangular R matrix */
     if(Rsolve<0)
       {uval=Rsolve;
       result=Rsolve;
       }
     else
       {

	 /* 3. calculate A, B, C, D */
	 xdif_pos=xtab[pos+1]-xtab[pos];
	 A=(xtab[pos+1]-x)/xdif_pos;
	 B=1.0-A;
	 C=(A*A*A-A)*xdif_pos*xdif_pos/6.0;
	 D=(B*B*B-B)*xdif_pos*xdif_pos/6.0;
 
	 /* 4. calculate yhat with standard spline form, store in the u parameter to the function */
	 u = A*ytab[pos] + B*ytab[pos+1] + C*ycoef[pos] + D*ycoef[pos+1];

	 /* 5. calculate sensitivity terms and root-sum-square 
	    a. calculate partials of ycoef[pos] with respect to ytab[i] and store in ypp_pos_coef
	    b. do the same for ypp_pos1_coef
	    c. calculate overall coefs to calculate yhat and put into ycoef
	    d. recalculate yhat based on coef's (as a basic derivation verification)
	    e. construct yuncertainty estimate on yhat
           
	    WARNING: this a potential hot spot. This loop runs in order n*n, and can be
	    replaced with one that runs in order n if a tridiagonal LU factorization is performed.
	 */
	 for(int k=0;k<ny;k++)
	   {trigetrow(S_a, S_b, S_c,  rprod, k, ny);
	   Rsolve=tridiagsolve(R_a, R_b, R_c, rprod, ycoef, gam, 1E-6, ny);
	   ypp_pos_coef[k]=ycoef[pos];
	   ypp_pos1_coef[k]=ycoef[pos+1];
	   }
     
     
	 for(int k=0;k<ny;k++)
	   ycoef[k]=C*ypp_pos_coef[k] + D*ypp_pos1_coef[k];
	 ycoef[pos]+=A;
	 ycoef[pos+1]+=B;
  
	 double yhatsigmasq =0;
	 double yhat=0;
  
	 for(int k=0;k<ny;k++)
	   {yhatsigmasq += ycoef[k]*ycoef[k]*ysigma[k]*ysigma[k];
	   yhat+= ycoef[k]*ytab[k];
	   }
  
	 usigma=sqrt(yhatsigmasq);
       }
   }
 if ((uval!=uprop_normal))
   umask|= pubdata.dmask_illdef;
 else
   umask&=~pubdata.dmask_illdef;
        
 delete []R_a;
 delete []R_b;
 delete []R_c;
 delete []S_a;
 delete []S_b;
 delete []S_c;
 delete []gam;
 delete []ycoef;
 delete []rprod;
 delete []ypp_pos_coef;
 delete []ypp_pos1_coef;
 
 return result;
}


/* 

   Double Precision Variant

 */

int upropclass:: interp(double & u, double & usigma, int & umask, double x, int order, int ny, double *xtab,double *ytab, double *ysigma)
{int result;
 if(ny<=0)
   {u=const_nan_float.f;
   usigma=const_nan_float.f;
   return -1;
   }
 else if(ny<=1)
   { u=ytab[0];
   usigma= const_nan_float.f;
   return -2;
   } 
 else if(ny==2)
   { result=linear_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
 else if((order<=2) || (x <xtab[0]) ||(x>xtab[ny-1]))
   { result=linear_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
 else 
   { result=spline_interp(u,usigma,umask,x,ny,xtab,ytab,ysigma);
   return result;
   }
}
/* the constant is in Taylor: Error Analysis, pp156 */
const double fwhm_to_sigma=2*sqrt(2*log((double)2));
double upropclass::EquivSigma(double Cauchy_fwhm)
{return Cauchy_fwhm/fwhm_to_sigma /*2*sqrt(2*ln(2))*/;
}

 
/* linear interpolation
    linear extrapolation when x is outside the tabulated range
 */

int upropclass:: linear_interp(double & u, double & usigma, int & umask, double x, int ny, double *xtab,double *ytab, double *ysigma)
{int result,pos,hi,lo,uval;
 double ratio, tsigmasq;
 
 uval=0;
 result=0;
 
 /* look for abnormal boundary conditions */
 if(ny<2) 
   {result=-3;
   u=const_nan_float.f;
   usigma=const_nan_float.f;
   uval=uprop_missing;
   }
 else if(x<xtab[0])
   { /* extrapolation */
     pos=0;
     ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
     u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
     tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
     usigma=sqrt(tsigmasq);
     uval=pubdata.dmask_warning;
     result=1;
   }
 else if(x>xtab[ny-1])
   { /* extrapolation */
     pos=ny-2;
     ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
     u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
     tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
     usigma=sqrt(tsigmasq);
     uval=pubdata.dmask_warning;
     result=1;
   }
 else
   {
     /* binary search for table location  */
     lo=-1;
     hi=ny;
     while ((hi-lo)>1)
       {pos=(lo+hi)>>1;
       if(xtab[pos]<=x)
	 lo=pos;
       else
	 hi=pos;
       }
     pos=lo;
     if(pos==(ny-1))
       pos=ny-2;
     /* linear interpolation */
     if((xtab[pos]<=x) && (x <= xtab[pos+1])  )
       {if(xtab[pos]<xtab[pos+1])
	 {ratio=(x-xtab[pos])/(xtab[pos+1]-xtab[pos]);
	 u=ytab[pos]*(1.0-ratio) + (ytab[pos+1]*ratio);
	 tsigmasq = ysigma[pos]*ysigma[pos]*(1.0-ratio)*(1.0-ratio)  +  ysigma[pos+1]*ysigma[pos+1]*ratio*ratio;
	 usigma=sqrt(tsigmasq);
	 }
       else
	 {u=ytab[pos];
	 usigma=ysigma[pos];
	 result=0;
	 }
       }  
     else
       {u=const_nan_float.f;
       usigma=const_nan_float.f;
       uval=uprop_missing;
       result=-4;
       }
   }
 if ((uval!=uprop_normal))
   umask|= pubdata.dmask_illdef;
 else
   umask&=~pubdata.dmask_illdef;
 return result;
}
int upropclass::spline_interp(double & u, double & usigma, int & umask, double x, int ny, double *xtab,double *ytab, double *ysigma)
{int result,pos,hi,lo,uval,memerr, Rsolve;

 double A,B,C,D,xdif_ip1,xdif_i,xdif_pos;
   
 double  *R_a, *R_b,*R_c, *S_a, *S_b, *S_c, *gam, *ycoef, *rprod, *ypp_pos1_coef, *ypp_pos_coef ;
 
 /* initialize all auxillary array pointers to NIL for safe cleanup in case of memory exhaustion */
 R_a=R_b=R_c=S_a=S_b=S_c=gam=ycoef=rprod=ypp_pos1_coef=ypp_pos_coef=0;
 memerr=0;

 if(ny>0)
   {
     memerr |=( R_a=new double[ny])==0;
     memerr |=( R_b=new double[ny])==0;
     memerr |=( R_c=new double[ny])==0;
     memerr |=( S_a=new double[ny])==0;
     memerr |=( S_b=new double[ny])==0;
     memerr |=( S_c=new double[ny])==0;
     memerr |=( gam=new double[ny])==0;
     memerr |=( ycoef=new double[ny])==0;
     memerr |=( rprod=new double[ny])==0;
     memerr |=( ypp_pos1_coef=new double[ny])==0;
     memerr |=( ypp_pos_coef=new double[ny])==0;
   }
    
 uval=0;
 result=0;

 /* check for memory allocation error */
 if(memerr!=0)
   {result=-5;
   u=const_nan_double.f;
   usigma=const_nan_double.f;
   uval=uprop_missing;
   }
 
 /* look for abnormal boundary conditions */
 else if( (ny<3) ||  (x<xtab[0]) || (x>xtab[ny-1]))
   {result=-3;
   u=const_nan_float.f;
   usigma=const_nan_double.f;
   uval=uprop_missing;
   }
 else
   {
     /* binary search for table location  */
     lo=-1;
     hi=ny;
     while ((hi-lo)>1)
       {pos=(lo+hi+1)>>1;
       if(xtab[pos]<=x)
	 lo=pos;
       else
	 hi=pos;
       }
     pos=lo;
     if(pos==(ny-1))
       pos=ny-2;
     /* after search and boundary check, 
	0<= pos < ny-1  and xtab[pos] <= x <= x[pos+1] */



     /* spline interpolation 
	1. Calculate tridiagonal R and S matrices
        R y" = S y
	2. Solve for coefs of ypp_j and ypp_jp1
	3. Calculate A,B,C,D
	4. Calculate interpolated yhat
	5. Calculate uncertainty on yhat using coefs
     */

     /* 1. create the triangular-stored R and S matrices */
     ycoef[0]=1.0;
     ycoef[1]=0;
     dtriputrow(R_a, R_b, R_c, ycoef , 0, ny);
     ycoef[0]=0;
     ycoef[1]=0;
     dtriputrow(S_a, S_b, S_c, ycoef , 0, ny);

     for(int i=1;i<(ny-1);i++)
       { /* set rprod to be R row i */
	 xdif_i=xtab[i]-xtab[i-1];
	 xdif_ip1=xtab[i+1]-xtab[i];

	 ycoef[i-1]=xdif_i/6.0;
	 ycoef[i+1]=xdif_ip1/6.0;
	 ycoef[i]  = (xtab[i+1]-xtab[i-1])/3.0;
	 dtriputrow(R_a, R_b, R_c, ycoef , i, ny);

	 ycoef[i-1] = 1.0/xdif_i;
	 ycoef[i]= - 1.0/xdif_ip1 -1.0/xdif_i;
	 ycoef[i+1]=  1.0/xdif_ip1;
	 dtriputrow(S_a, S_b, S_c, ycoef , i, ny);
       }
     ycoef[ny-1]=1.0;
     ycoef[ny-2]=0;
     dtriputrow(R_a, R_b, R_c, ycoef , ny-1, ny);
     ycoef[ny-1]=0;
     ycoef[ny-2]=0;
     dtriputrow(S_a, S_b, S_c, ycoef , ny-1, ny);

     /* 2. Solve for the second derivatives , place in ycoef */
     dtrimulvec(S_a, S_b, S_c, ytab ,rprod, ny);
     Rsolve=dtridiagsolve(R_a, R_b, R_c, rprod, ycoef, gam, 1E-6, ny);
  
     /* check for singular or malformed triangular R matrix */
     if(Rsolve<0)
       {uval=Rsolve;
       result=Rsolve;
       }
     else
       {

	 /* 3. calculate A, B, C, D */
	 xdif_pos=xtab[pos+1]-xtab[pos];
	 A=(xtab[pos+1]-x)/xdif_pos;
	 B=1.0-A;
	 C=(A*A*A-A)*xdif_pos*xdif_pos/6.0;
	 D=(B*B*B-B)*xdif_pos*xdif_pos/6.0;
 
	 /* 4. calculate yhat with standard spline form, store in the u parameter to the function */
	 u = A*ytab[pos] + B*ytab[pos+1] + C*ycoef[pos] + D*ycoef[pos+1];

	 /* 5. calculate sensitivity terms and root-sum-square 
	    a. calculate partials of ycoef[pos] with respect to ytab[i] and store in ypp_pos_coef
	    b. do the same for ypp_pos1_coef
	    c. calculate overall coefs to calculate yhat and put into ycoef
	    d. recalculate yhat based on coef's (as a basic derivation verification)
	    e. construct yuncertainty estimate on yhat
           
	    WARNING: this a potential hot spot. This loop runs in order n*n, and can be
	    replaced with one that runs in order n if a tridiagonal LU factorization is performed.
	 */
	 for(int k=0;k<ny;k++)
	   {dtrigetrow(S_a, S_b, S_c,  rprod, k, ny);
	   Rsolve=dtridiagsolve(R_a, R_b, R_c, rprod, ycoef, gam, 1E-6, ny);
	   ypp_pos_coef[k]=ycoef[pos];
	   ypp_pos1_coef[k]=ycoef[pos+1];
	   }
     
     
	 for(int k=0;k<ny;k++)
	   ycoef[k]=C*ypp_pos_coef[k] + D*ypp_pos1_coef[k];
	 ycoef[pos]+=A;
	 ycoef[pos+1]+=B;
  
	 double yhatsigmasq =0;
	 double yhat=0;
  
	 for(int k=0;k<ny;k++)
	   {yhatsigmasq += ycoef[k]*ycoef[k]*ysigma[k]*ysigma[k];
	   yhat+= ycoef[k]*ytab[k];
	   }
  
	 usigma=sqrt(yhatsigmasq);
       }
   }
 if ((uval!=uprop_normal))
   umask|= pubdata.dmask_illdef;
 else
   umask&=~pubdata.dmask_illdef;
        
 delete []R_a;
 delete []R_b;
 delete []R_c;
 delete []S_a;
 delete []S_b;
 delete []S_c;
 delete []gam;
 delete []ycoef;
 delete []rprod;
 delete []ypp_pos_coef;
 delete []ypp_pos1_coef;
 
 return result;
}

